@@ -22,10 +22,9 @@
 
 #include "appitem.h"
 #include "themeappicon.h"
-#include "imagefactory.h"
 #include "xcb_misc.h"
 #include "appswingeffectbuilder.h"
-#include "appspreviewprovider.h"
+#include "utils.h"
 
 #include <X11/X.h>
 #include <X11/Xlib.h>
@@ -41,6 +40,10 @@
 #include <QGSettings>
 
 #include <DGuiApplicationHelper>
+#include <DConfig>
+
+DGUI_USE_NAMESPACE
+DCORE_USE_NAMESPACE
 
 #define APP_DRAG_THRESHOLD      20
 
@@ -51,7 +54,6 @@ AppItem::AppItem(const QGSettings *appSettings, const QGSettings *activeAppSetti
     , m_appSettings(appSettings)
     , m_activeAppSettings(activeAppSettings)
     , m_dockedAppSettings(dockedAppSettings)
-    , m_appNameTips(new TipsWidget(this))
     , m_appPreviewTips(nullptr)
     , m_itemEntryInter(new DockEntryInter("com.deepin.dde.daemon.Dock", entry.path(), QDBusConnection::sessionBus(), this))
     , m_swingEffectView(nullptr)
@@ -60,14 +62,13 @@ AppItem::AppItem(const QGSettings *appSettings, const QGSettings *activeAppSetti
     , m_drag(nullptr)
     , m_dragging(false)
     , m_retryTimes(0)
-    , m_lastShowDay(0)
+    , m_iconValid(true)
     , m_lastclickTimes(0)
     , m_appIcon(QPixmap())
     , m_updateIconGeometryTimer(new QTimer(this))
     , m_retryObtainIconTimer(new QTimer(this))
     , m_refershIconTimer(new QTimer(this))
-    , m_smallWatcher(new QFutureWatcher<QPixmap>(this))
-    , m_largeWatcher(new QFutureWatcher<QPixmap>(this))
+    , m_themeType(DGuiApplicationHelper::instance()->themeType())
 {
     QHBoxLayout *centralLayout = new QHBoxLayout;
     centralLayout->setMargin(0);
@@ -80,14 +81,10 @@ AppItem::AppItem(const QGSettings *appSettings, const QGSettings *activeAppSetti
     m_id = m_itemEntryInter->id();
     m_active = m_itemEntryInter->isActive();
 
-    m_appNameTips->setObjectName(m_itemEntryInter->name());
-    m_appNameTips->setVisible(false);
-    m_appNameTips->installEventFilter(this);
-
     m_updateIconGeometryTimer->setInterval(500);
     m_updateIconGeometryTimer->setSingleShot(true);
 
-    m_retryObtainIconTimer->setInterval(500);
+    m_retryObtainIconTimer->setInterval(3000);
     m_retryObtainIconTimer->setSingleShot(true);
 
     m_refershIconTimer->setInterval(1000);
@@ -115,22 +112,13 @@ AppItem::AppItem(const QGSettings *appSettings, const QGSettings *activeAppSetti
 
     connect(DGuiApplicationHelper::instance(), &DGuiApplicationHelper::themeTypeChanged, this, &AppItem::onThemeTypeChanged);
 
-    connect(m_refershIconTimer, &QTimer::timeout, this, [ = ]() {
-        m_curDate = QDate::currentDate();
-        if (m_curDate.day() != m_lastShowDay) {
-            m_lastShowDay = m_curDate.day();
-            refreshIcon();
-        }
-    });
-}
-
-AppItem::~AppItem()
-{
-    stopSwingEffect();
-
-    m_appNameTips->deleteLater();
+    /** 日历 1S定时判断是否刷新icon的处理 */
+    connect(m_refershIconTimer, &QTimer::timeout, this, &AppItem::onRefreshIcon);
 }
 
+/**将属于同一个应用的窗口合并到同一个应用图标
+ * @brief AppItem::checkEntry
+ */
 void AppItem::checkEntry()
 {
     m_itemEntryInter->Check();
@@ -151,6 +139,10 @@ bool AppItem::isValid() const
 // window behaviors like minimization.
 void AppItem::updateWindowIconGeometries()
 {
+    // wayland没做处理
+    if (Utils::IS_WAYLAND_DISPLAY)
+        return;
+
     const QRect r(mapToGlobal(QPoint(0, 0)),
                   mapToGlobal(QPoint(width(), height())));
     if (!QX11Info::connection()) {
@@ -164,6 +156,9 @@ void AppItem::updateWindowIconGeometries()
         xcb_misc->set_window_icon_geometry(it.key(), r);
 }
 
+/**取消驻留在dock上的应用
+ * @brief AppItem::undock
+ */
 void AppItem::undock()
 {
     m_itemEntryInter->RequestUndock();
@@ -300,7 +295,8 @@ void AppItem::mouseReleaseEvent(QMouseEvent *e)
         return;
     }
 
-    int curTimestamp = QX11Info::getTimestamp();
+    // 获取时间戳qint64转quint64，是不存在任何问题的
+    quint64 curTimestamp = QDateTime::currentDateTime().toMSecsSinceEpoch();
     if ((curTimestamp - m_lastclickTimes) < 300)
         return;
 
@@ -330,7 +326,7 @@ void AppItem::mouseReleaseEvent(QMouseEvent *e)
         m_itemEntryInter->Activate(QX11Info::getTimestamp());
 
         // play launch effect
-        if (m_windowInfos.isEmpty())
+        if (m_windowInfos.isEmpty() && DGuiApplicationHelper::isSpecialEffectsEnvironment())
             playSwingEffect();
     }
 }
@@ -448,17 +444,6 @@ void AppItem::leaveEvent(QEvent *e)
     }
 }
 
-void AppItem::showEvent(QShowEvent *e)
-{
-    DockItem::showEvent(e);
-
-    QTimer::singleShot(0, this, [ = ] {
-        onGSettingsChanged("enable");
-    });
-
-    refreshIcon();
-}
-
 void AppItem::showHoverTips()
 {
     if (checkGSettingsControl()) {
@@ -485,22 +470,25 @@ const QString AppItem::contextMenu() const
 
 QWidget *AppItem::popupTips()
 {
-    if (checkGSettingsControl()) {
+    if (checkGSettingsControl())
         return nullptr;
-    }
 
     if (m_dragging)
         return nullptr;
 
+    static TipsWidget appNameTips(topLevelWidget());
+    appNameTips.setAccessibleName("tip");
+    appNameTips.setObjectName(m_itemEntryInter->name());
+
     if (!m_windowInfos.isEmpty()) {
         const quint32 currentWindow = m_itemEntryInter->currentWindow();
         Q_ASSERT(m_windowInfos.contains(currentWindow));
-        m_appNameTips->setText(m_windowInfos[currentWindow].title.simplified());
+        appNameTips.setText(m_windowInfos[currentWindow].title.simplified());
     } else {
-        m_appNameTips->setText(m_itemEntryInter->name().simplified());
+        appNameTips.setText(m_itemEntryInter->name().simplified());
     }
 
-    return m_appNameTips;
+    return &appNameTips;
 }
 
 void AppItem::startDrag()
@@ -556,10 +544,11 @@ void AppItem::startDrag()
 
 bool AppItem::hasAttention() const
 {
-    for (const auto &info : m_windowInfos)
-        if (info.attention)
-            return true;
-    return false;
+    auto it = std::find_if(m_windowInfos.constBegin(), m_windowInfos.constEnd(), [ = ] (const auto &info) {
+        return info.attention;
+    });
+
+    return (it != m_windowInfos.end());
 }
 
 QPoint AppItem::appIconPosition() const
@@ -600,20 +589,31 @@ void AppItem::refreshIcon()
     const int iconSize = qMin(width(), height());
 
     if (DockDisplayMode == Efficient)
-        m_appIcon = ThemeAppIcon::getIcon(icon, iconSize * 0.7, devicePixelRatioF());
+        m_iconValid = ThemeAppIcon::getIcon(m_appIcon, icon, iconSize * 0.7, !m_iconValid);
     else
-        m_appIcon = ThemeAppIcon::getIcon(icon, iconSize * 0.8, devicePixelRatioF());
+        m_iconValid = ThemeAppIcon::getIcon(m_appIcon, icon, iconSize * 0.8, !m_iconValid);
 
     if (!m_refershIconTimer->isActive() && m_itemEntryInter->icon() == "dde-calendar") {
         m_refershIconTimer->start();
     }
 
-    if (m_appIcon.isNull()) {
-        if (m_retryTimes < 5) {
+    if (!m_iconValid) {
+        if (m_retryTimes < 10) {
             m_retryTimes++;
             qDebug() << m_itemEntryInter->name() << "obtain app icon(" << icon << ")failed, retry times:" << m_retryTimes;
+            // Maybe the icon was installed after we loaded the caches.
+            // QIcon::setThemeSearchPaths will force Qt to re-check the gtk cache validity.
+            QIcon::setThemeSearchPaths(QIcon::themeSearchPaths());
+
             m_retryObtainIconTimer->start();
+        } else {
+            // 如果图标获取失败，一分钟后再自动刷新一次（如果还是显示异常，基本需要应用自身看下为什么了）
+            if (!m_iconValid)
+                QTimer::singleShot(60 * 1000, this, &AppItem::refreshIcon);
         }
+
+        update();
+
         return;
     } else if (m_retryTimes > 0) {
         // reset times
@@ -625,6 +625,23 @@ void AppItem::refreshIcon()
     m_updateIconGeometryTimer->start();
 }
 
+void AppItem::onRefreshIcon()
+{
+    if (QDate::currentDate() == m_curDate)
+        return;
+
+    m_curDate = QDate::currentDate();
+    refreshIcon();
+}
+
+void AppItem::onResetPreview()
+{
+    if (m_appPreviewTips != nullptr) {
+        m_appPreviewTips->deleteLater();
+        m_appPreviewTips = nullptr;
+    }
+}
+
 void AppItem::activeChanged()
 {
     m_active = !m_active;
@@ -635,7 +652,10 @@ void AppItem::showPreview()
     if (m_windowInfos.isEmpty())
         return;
 
-    m_appPreviewTips = PreviewWindow(m_windowInfos, m_itemEntryInter->GetAllowedCloseWindows().value(), DockPosition);
+    m_appPreviewTips = new PreviewContainer;
+
+    m_appPreviewTips->updateLayoutDirection(DockPosition);
+    m_appPreviewTips->setWindowInfos(m_windowInfos, m_itemEntryInter->GetAllowedCloseWindows().value());
 
     connect(m_appPreviewTips, &PreviewContainer::requestActivateWindow, this, &AppItem::requestActivateWindow, Qt::QueuedConnection);
     connect(m_appPreviewTips, &PreviewContainer::requestPreviewWindow, this, &AppItem::requestPreviewWindow, Qt::QueuedConnection);
@@ -643,9 +663,14 @@ void AppItem::showPreview()
     connect(m_appPreviewTips, &PreviewContainer::requestHidePopup, this, &AppItem::hidePopup);
     connect(m_appPreviewTips, &PreviewContainer::requestCheckWindows, m_itemEntryInter, &DockEntryInter::Check);
 
-    connect(m_appPreviewTips, &PreviewContainer::requestActivateWindow, [ = ]() { m_appPreviewTips = nullptr; });
-    connect(m_appPreviewTips, &PreviewContainer::requestCancelPreviewWindow, [ = ]() { m_appPreviewTips = nullptr; });
-    connect(m_appPreviewTips, &PreviewContainer::requestHidePopup, [ = ]() { m_appPreviewTips = nullptr; });
+    connect(m_appPreviewTips, &PreviewContainer::requestActivateWindow, this, &AppItem::onResetPreview);
+    connect(m_appPreviewTips, &PreviewContainer::requestCancelPreviewWindow, this, &AppItem::onResetPreview);
+    connect(m_appPreviewTips, &PreviewContainer::requestHidePopup, this, &AppItem::onResetPreview);
+
+    // 预览标题显示方式的配置
+    DConfig config(QString("com.deepin.dde.dock.dconfig"), QString());
+    if (config.isValid() && config.keyList().contains("Dock_Show_Window_name"))
+        m_appPreviewTips->setTitleDisplayMode(config.value("Dock_Show_Window_name").toInt());
 
     showPopupWindow(m_appPreviewTips, true);
 }
@@ -656,6 +681,9 @@ void AppItem::playSwingEffect()
     if (m_swingEffectView != nullptr)
         return;
 
+    if (rect().isEmpty())
+        return checkAttentionEffect();
+
     stopSwingEffect();
 
     QPair<QGraphicsView *, QGraphicsItemAnimation *> pair =  SwingEffect(
@@ -665,7 +693,7 @@ void AppItem::playSwingEffect()
     m_itemAnimation = pair.second;
 
     QTimeLine *tl = m_itemAnimation->timeLine();
-    connect(tl, &QTimeLine::stateChanged, [ = ](QTimeLine::State newState) {
+    connect(tl, &QTimeLine::stateChanged, this, [ = ](QTimeLine::State newState) {
         if (newState == QTimeLine::NotRunning) {
             m_swingEffectView->hide();
             layout()->removeWidget(m_swingEffectView);
@@ -721,8 +749,8 @@ bool AppItem::checkGSettingsControl() const
             ? m_dockedAppSettings
             : m_activeAppSettings;
 
-    return (setting && setting->keys().contains("control") && setting->get("control").toBool()) ||
-            (m_appSettings && m_appSettings->keys().contains("control") && m_appSettings->get("control").toBool());
+    return ((m_appSettings && m_appSettings->keys().contains("control") && m_appSettings->get("control").toBool())
+            || (setting && setting->keys().contains("control") && setting->get("control").toBool()));
 }
 
 void AppItem::onThemeTypeChanged(DGuiApplicationHelper::ColorType themeType)
@@ -730,3 +758,20 @@ void AppItem::onThemeTypeChanged(DGuiApplicationHelper::ColorType themeType)
     m_themeType = themeType;
     update();
 }
+
+// 放到最下面是因为析构函数和匿名函数会影响lcov统计单元测试的覆盖率
+AppItem::~AppItem()
+{
+    stopSwingEffect();
+}
+
+void AppItem::showEvent(QShowEvent *e)
+{
+    DockItem::showEvent(e);
+
+    QTimer::singleShot(0, this, [ = ] {
+        onGSettingsChanged("enable");
+    });
+
+    refreshIcon();
+}
