@@ -20,12 +20,12 @@
  */
 
 #include "dockitem.h"
-#include "hoverhighlighteffect.h"
 #include "pluginsitem.h"
 
 #include <QMouseEvent>
 #include <QJsonObject>
 #include <QCursor>
+#include <QApplication>
 
 #define PLUGIN_MARGIN  10
 #define ITEM_MAXSIZE    100
@@ -40,11 +40,8 @@ DockItem::DockItem(QWidget *parent)
     , m_popupShown(false)
     , m_tapAndHold(false)
     , m_draging(false)
-    //FIXME: 可能是qt的bug，概率性导致崩溃，待修复
-//    , m_hoverEffect(new HoverHighlightEffect(this))
     , m_popupTipsDelayTimer(new QTimer(this))
     , m_popupAdjustDelayTimer(new QTimer(this))
-
 {
     if (PopupWindow.isNull()) {
         DockPopupWindow *arrowRectangle = new DockPopupWindow(nullptr);
@@ -56,6 +53,7 @@ DockItem::DockItem(QWidget *parent)
         arrowRectangle->setArrowHeight(10);
         arrowRectangle->setObjectName("apppopup");
         PopupWindow = arrowRectangle;
+        connect(qApp, &QApplication::aboutToQuit, PopupWindow, &DockPopupWindow::deleteLater);
     }
 
     m_popupTipsDelayTimer->setInterval(500);
@@ -63,8 +61,6 @@ DockItem::DockItem(QWidget *parent)
 
     m_popupAdjustDelayTimer->setInterval(10);
     m_popupAdjustDelayTimer->setSingleShot(true);
-//FIXME: 可能是qt的bug，概率性导致崩溃，待修复
-//    setGraphicsEffect(m_hoverEffect);
 
     connect(m_popupTipsDelayTimer, &QTimer::timeout, this, &DockItem::showHoverTips);
     connect(m_popupAdjustDelayTimer, &QTimer::timeout, this, &DockItem::updatePopupPosition, Qt::QueuedConnection);
@@ -75,11 +71,15 @@ DockItem::DockItem(QWidget *parent)
     setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
 }
 
+DockItem::~DockItem()
+{
+    if (m_popupShown)
+        popupWindowAccept();
+}
+
 QSize DockItem::sizeHint() const
 {
-    int size = qMin(maximumWidth(), maximumHeight());
-    if (size > ITEM_MAXSIZE)
-        size = ITEM_MAXSIZE;
+    int size = qMin(qMin(maximumWidth(), maximumHeight()), ITEM_MAXSIZE);
 
     return QSize(size, size);
 }
@@ -89,12 +89,6 @@ QString DockItem::accessibleName()
     return QString();
 }
 
-DockItem::~DockItem()
-{
-    if (m_popupShown)
-        popupWindowAccept();
-}
-
 void DockItem::setDockPosition(const Position side)
 {
     DockPosition = side;
@@ -255,7 +249,7 @@ void DockItem::showContextMenu()
     hidePopup();
     emit requestWindowAutoHide(false);
 
-    m_contextMenu.popup(QCursor::pos());
+    m_contextMenu.exec(QCursor::pos());
 
     onContextMenuAccepted();
 }
