@@ -29,6 +29,8 @@
 #include "dockitemmanager.h"
 #include "touchsignalmanager.h"
 #include "utils.h"
+#include "desktop_widget.h"
+#include "imageutil.h"
 
 #include <QDrag>
 #include <QTimer>
@@ -36,6 +38,17 @@
 #include <QString>
 #include <QApplication>
 #include <QPointer>
+#include <QBoxLayout>
+#include <QLabel>
+#include <QPixmap>
+#include <QtConcurrent/QtConcurrentRun>
+#include <qpa/qplatformnativeinterface.h>
+#include <qpa/qplatformintegration.h>
+#define protected public
+#include <QtGui/private/qsimpledrag_p.h>
+#undef protected
+#include <QtGui/private/qshapedpixmapdndwindow_p.h>
+#include <QtGui/private/qguiapplication_p.h>
 
 #include <DGuiApplicationHelper>
 #include <DWindowManagerHelper>
@@ -52,36 +65,34 @@ MainPanelControl::MainPanelControl(QWidget *parent)
     : QWidget(parent)
     , m_mainPanelLayout(new QBoxLayout(QBoxLayout::LeftToRight, this))
     , m_fixedAreaWidget(new QWidget(this))
-    , m_fixedAreaLayout(new QBoxLayout(QBoxLayout::LeftToRight))
+    , m_fixedAreaLayout(new QBoxLayout(QBoxLayout::LeftToRight, this))
     , m_fixedSpliter(new QLabel(this))
     , m_appAreaWidget(new QWidget(this))
     , m_appAreaSonWidget(new QWidget(this))
-    , m_appAreaSonLayout(new QBoxLayout(QBoxLayout::LeftToRight))
+    , m_appAreaSonLayout(new QBoxLayout(QBoxLayout::LeftToRight, this))
     , m_appSpliter(new QLabel(this))
     , m_trayAreaWidget(new QWidget(this))
-    , m_trayAreaLayout(new QBoxLayout(QBoxLayout::LeftToRight))
+    , m_trayAreaLayout(new QBoxLayout(QBoxLayout::LeftToRight, this))
     , m_traySpliter(new QLabel(this))
     , m_pluginAreaWidget(new QWidget(this))
-    , m_pluginLayout(new QBoxLayout(QBoxLayout::LeftToRight))
+    , m_pluginLayout(new QBoxLayout(QBoxLayout::LeftToRight, this))
     , m_desktopWidget(new DesktopWidget(this))
     , m_position(Position::Bottom)
     , m_placeholderItem(nullptr)
     , m_appDragWidget(nullptr)
     , m_dislayMode(Efficient)
-    , m_isHover(false)
-    , m_needRecoveryWin(false)
+    , m_tray(nullptr)
+    , m_trashItem(nullptr)
 {
-    initUi();
+    initUI();
     updateMainPanelLayout();
     setAcceptDrops(true);
     setMouseTracking(true);
-    m_desktopWidget->setMouseTracking(true);
-    m_desktopWidget->setObjectName("showdesktoparea");
 
     m_appAreaWidget->installEventFilter(this);
     m_appAreaSonWidget->installEventFilter(this);
     m_trayAreaWidget->installEventFilter(this);
-    m_desktopWidget->installEventFilter(this);
+    m_pluginAreaWidget->installEventFilter(this);
 
     //在设置每条线大小前，应该设置fixedsize(0,0)
     //应为paintEvent函数会先调用设置背景颜色，大小为随机值
@@ -90,11 +101,7 @@ MainPanelControl::MainPanelControl(QWidget *parent)
     m_traySpliter->setFixedSize(0,0);
 }
 
-MainPanelControl::~MainPanelControl()
-{
-}
-
-void MainPanelControl::initUi()
+void MainPanelControl::initUI()
 {
     /* 固定区域 */
     m_fixedAreaWidget->setObjectName("fixedarea");
@@ -132,9 +139,10 @@ void MainPanelControl::initUi()
     m_pluginAreaWidget->setLayout(m_pluginLayout);
     m_pluginLayout->setSpacing(10);
     m_pluginLayout->setContentsMargins(0, 0, 0, 0);
-    m_mainPanelLayout->addWidget(m_pluginAreaWidget);
+    m_mainPanelLayout->addWidget(m_pluginAreaWidget, 0, Qt::AlignCenter);
 
     /* 桌面预览 */
+    m_desktopWidget->setObjectName("showdesktoparea");
     m_mainPanelLayout->addWidget(m_desktopWidget);
 
     m_mainPanelLayout->setSpacing(0);
@@ -144,6 +152,10 @@ void MainPanelControl::initUi()
     m_mainPanelLayout->setAlignment(m_traySpliter, Qt::AlignCenter);
 }
 
+/**
+ * @brief MainPanelControl::setDisplayMode 根据任务栏显示模式更新界面显示，如果是时尚模式，没有‘显示桌面'区域，否则就有
+ * @param dislayMode 任务栏显示模式
+ */
 void MainPanelControl::setDisplayMode(DisplayMode dislayMode)
 {
     if (dislayMode == m_dislayMode)
@@ -152,6 +164,9 @@ void MainPanelControl::setDisplayMode(DisplayMode dislayMode)
     updateDisplayMode();
 }
 
+/**根据任务栏在屏幕上的位置，更新任务栏各控件布局
+ * @brief MainPanelControl::updateMainPanelLayout
+ */
 void MainPanelControl::updateMainPanelLayout()
 {
     switch (m_position) {
@@ -185,10 +200,18 @@ void MainPanelControl::updateMainPanelLayout()
         break;
     }
 
+    // 显示桌面的区域
     resizeDesktopWidget();
+
+    // 设置任务栏各区域图标大小
     resizeDockIcon();
 }
 
+/**往固定区域添加应用
+ * @brief MainPanelControl::addFixedAreaItem
+ * @param index　位置索引，如果为负数则插入到最后，为正则插入到指定位置
+ * @param wdg　应用指针对象
+ */
 void MainPanelControl::addFixedAreaItem(int index, QWidget *wdg)
 {
     if(m_position == Position::Top || m_position == Position::Bottom){
@@ -199,6 +222,11 @@ void MainPanelControl::addFixedAreaItem(int index, QWidget *wdg)
     m_fixedAreaLayout->insertWidget(index, wdg);
 }
 
+/**往应用区域添加应用
+ * @brief MainPanelControl::addAppAreaItem
+ * @param index　位置索引，如果为负数则插入到最后，为正则插入到指定位置
+ * @param wdg　应用指针对象
+ */
 void MainPanelControl::addAppAreaItem(int index, QWidget *wdg)
 {
     if(m_position == Position::Top || m_position == Position::Bottom){
@@ -209,57 +237,101 @@ void MainPanelControl::addAppAreaItem(int index, QWidget *wdg)
     m_appAreaSonLayout->insertWidget(index, wdg);
 }
 
+/**往托盘插件区域添加应用
+ * @brief MainPanelControl::addTrayAreaItem
+ * @param index　位置索引，如果为负数则插入到最后，为正则插入到指定位置
+ * @param wdg　应用指针对象
+ */
 void MainPanelControl::addTrayAreaItem(int index, QWidget *wdg)
 {
     m_tray = static_cast<TrayPluginItem *>(wdg);
     m_trayAreaLayout->insertWidget(index, wdg);
 }
 
+/**往插件区域添加应用，保存回收站插件指针对象
+ * @brief MainPanelControl::addPluginAreaItem
+ * @param index　位置索引，如果为负数则插入到最后，为正则插入到指定位置
+ * @param wdg　应用指针对象
+ */
 void MainPanelControl::addPluginAreaItem(int index, QWidget *wdg)
 {
     //因为日期时间插件和其他插件的大小有异，为了方便设置边距，在插件区域布局再添加一层布局设置边距
     //因此在处理插件图标时，需要通过两层布局判断是否为需要的插件，例如拖动插件位置等判断
-    QBoxLayout * boxLayout = new QBoxLayout(QBoxLayout::LeftToRight);
+    QBoxLayout * boxLayout = new QBoxLayout(QBoxLayout::LeftToRight, this);
     boxLayout->addWidget(wdg, 0, Qt::AlignCenter);
     m_pluginLayout->insertLayout(index, boxLayout, 0);
+
+    // 保存垃圾箱插件指针
+    PluginsItem *pluginsItem = qobject_cast<PluginsItem *>(wdg);
+    if (pluginsItem && pluginsItem->pluginName() == "trash")
+        m_trashItem = pluginsItem;
 }
 
+/**移除固定区域某一应用
+ * @brief MainPanelControl::removeFixedAreaItem
+ * @param wdg 应用指针对象
+ */
 void MainPanelControl::removeFixedAreaItem(QWidget *wdg)
 {
     m_fixedAreaLayout->removeWidget(wdg);
 }
 
+/**移除应用区域某一应用
+ * @brief MainPanelControl::removeAppAreaItem
+ * @param wdg 应用指针对象
+ */
 void MainPanelControl::removeAppAreaItem(QWidget *wdg)
 {
     m_appAreaSonLayout->removeWidget(wdg);
 }
 
+/**移除托盘插件区域某一应用
+ * @brief MainPanelControl::removeTrayAreaItem
+ * @param wdg 应用指针对象
+ */
 void MainPanelControl::removeTrayAreaItem(QWidget *wdg)
 {
     m_trayAreaLayout->removeWidget(wdg);
 }
 
+/**移除插件区域某一应用
+ * @brief MainPanelControl::removePluginAreaItem
+ * @param wdg 应用指针对象
+ */
 void MainPanelControl::removePluginAreaItem(QWidget *wdg)
 {
     //因为日期时间插件大小和其他插件有异，为了方便设置边距，各插件中增加一层布局
     //因此remove插件图标时，需要从多的一层布局中取widget进行判断是否需要移除的插件
+    // 清空保存的垃圾箱插件指针
+    PluginsItem *pluginsItem = qobject_cast<PluginsItem *>(wdg);
+    if (pluginsItem && pluginsItem->pluginName() == "trash")
+        m_trashItem = nullptr;
+
     for (int i = 0; i < m_pluginLayout->count(); ++i) {
         QLayoutItem *layoutItem = m_pluginLayout->itemAt(i);
         QLayout *boxLayout = layoutItem->layout();
         if (boxLayout && boxLayout->itemAt(0)->widget() == wdg) {
             boxLayout->removeWidget(wdg);
             m_pluginLayout->removeItem(layoutItem);
+            delete layoutItem;
+            layoutItem = nullptr;
         }
     }
 }
 
 void MainPanelControl::resizeEvent(QResizeEvent *event)
 {
+    //先通过消息循环让各部件调整好size后再计算图标大小
+    //避免因为部件size没有调整完导致计算的图标大小不准确
+    //然后重复触发m_pluginAreaWidget的reszie事件并重复计算，造成任务栏图标抖动问题
+    QWidget::resizeEvent(event);
     resizeDesktopWidget();
     resizeDockIcon();
-    return QWidget::resizeEvent(event);
 }
 
+/**根据任务栏所在位置， 设置应用区域控件的大小
+ * @brief MainPanelControl::updateAppAreaSonWidgetSize
+ */
 void MainPanelControl::updateAppAreaSonWidgetSize()
 {
     if ((m_position == Position::Top) || (m_position == Position::Bottom)) {
@@ -275,19 +347,29 @@ void MainPanelControl::updateAppAreaSonWidgetSize()
     moveAppSonWidget();
 }
 
+/**
+ * @brief setPositonValue 根据传入的位置更新界面布局，比如任务栏在左，布局应该是上下布局，任务栏在下，应该是左右布局
+ * @param position 任务栏的位置
+ */
 void MainPanelControl::setPositonValue(Dock::Position position)
 {
     if (m_position == position)
         return;
 
     m_position = position;
-    QTimer::singleShot(0, this, [=] {
-        updateMainPanelLayout();
-    });
+    QTimer::singleShot(0, this, &MainPanelControl::updateMainPanelLayout);
 }
 
+/**向任务栏插入各类应用,并将属于同一个应用的窗口合并到同一个应用图标
+ * @brief MainPanelControl::insertItem
+ * @param index 位置索引
+ * @param item 应用指针对象
+ */
 void MainPanelControl::insertItem(int index, DockItem *item)
 {
+    if (!item)
+        return;
+
     item->installEventFilter(this);
 
     switch (item->itemType()) {
@@ -301,22 +383,26 @@ void MainPanelControl::insertItem(int index, DockItem *item)
     case DockItem::Placeholder:
         addAppAreaItem(index, item);
         break;
-    case DockItem::TrayPlugin:
+    case DockItem::TrayPlugin: // 此处只会有一个tray系统托盘插件，微信、声音、网络蓝牙等等，都在系统托盘插件中处理的
         addTrayAreaItem(index, item);
         break;
     case DockItem::Plugins:
         addPluginAreaItem(index, item);
         break;
-    default:
-        break;
     }
-    resizeDockIcon();
-    QTimer::singleShot(0, [ = ] {
-        updatePluginsLayout();
-    });
+
+    // 同removeItem处 注意:不能屏蔽此接口，否则会造成插件插入时无法显示
+    if (item->itemType() != DockItem::App)
+        resizeDockIcon();
+
     item->checkEntry();
+    QTimer::singleShot(0, this, &MainPanelControl::updatePluginsLayout);
 }
 
+/**从任务栏移除某一应用，并更新任务栏图标大小
+ * @brief MainPanelControl::removeItem
+ * @param item 应用指针对象
+ */
 void MainPanelControl::removeItem(DockItem *item)
 {
     switch (item->itemType()) {
@@ -334,22 +420,23 @@ void MainPanelControl::removeItem(DockItem *item)
     case DockItem::Plugins:
         removePluginAreaItem(item);
         break;
-    default:
-        break;
     }
-    resizeDockIcon();
-}
 
-MainPanelDelegate *MainPanelControl::delegate() const
-{
-    return m_delegate;
-}
+    item->removeEventFilter(this);
 
-void MainPanelControl::setDelegate(MainPanelDelegate *delegate)
-{
-    m_delegate = delegate;
+    /** 此处重新计算大小的时候icon的个数在原有个数上减少了一个，导致每个icon的大小跟原来大小不一致，需要重新设置setFixedSize
+     *  在龙芯处理器上当app数量过多时，会导致拖动app耗时严重，造成卡顿
+     *  注意:不能屏蔽此接口，否则会造成插件移除时无法更新icon大小
+     */
+    if (item->itemType() != DockItem::App)
+        resizeDockIcon();
 }
 
+/**任务栏移动应用图标
+ * @brief MainPanelControl::moveItem
+ * @param sourceItem 即将插入的应用
+ * @param targetItem 被移动的应用
+ */
 void MainPanelControl::moveItem(DockItem *sourceItem, DockItem *targetItem)
 {
     // get target index
@@ -473,7 +560,6 @@ void MainPanelControl::handleDragMove(QDragMoveEvent *e, bool isFilter)
         } else {
             targetItem = sourceItem;
         }
-
     } else {
         // other dockItem调整顺序
         targetItem = dropTargetItem(sourceItem, e->pos());
@@ -532,7 +618,7 @@ void MainPanelControl::dragMoveEvent(QDragMoveEvent *e)
         return;
     }
 
-    if (m_delegate && m_delegate->appIsOnDock(DragmineData->data(m_draggingMimeKey))) {
+    if (appIsOnDock(DragmineData->data(m_draggingMimeKey))) {
         e->setAccepted(false);
         return;
     }
@@ -542,6 +628,7 @@ void MainPanelControl::dragMoveEvent(QDragMoveEvent *e)
 
 bool MainPanelControl::eventFilter(QObject *watched, QEvent *event)
 {
+    // 更新应用区域大小和任务栏图标大小
     if (watched == m_appAreaSonWidget) {
         switch (event->type()) {
         case QEvent::LayoutRequest:
@@ -557,24 +644,18 @@ bool MainPanelControl::eventFilter(QObject *watched, QEvent *event)
         }
     }
 
-    if (watched == m_desktopWidget) {
-        if (event->type() == QEvent::Enter) {
-            if (checkNeedShowDesktop()) {
-                m_needRecoveryWin = true;
-                QProcess::startDetached("/usr/lib/deepin-daemon/desktop-toggle");
-            }
-            m_isHover = true;
-            update();
-        } else if (event->type() == QEvent::Leave) {
-            // 鼠标移入时隐藏了窗口，移出时恢复
-            if (m_needRecoveryWin) {
-                QProcess::startDetached("/usr/lib/deepin-daemon/desktop-toggle");
-            }
-            m_isHover = false;
-            update();
+    // fix:88133 在计算icon大小时m_pluginAreaWidget的数据错误
+    if (watched == m_pluginAreaWidget) {
+        switch (event->type()) {
+        case QEvent::Resize:
+            resizeDockIcon();
+            break;
+        default:
+            break;
         }
     }
 
+    // 更新应用区域子控件大小以及位置
     if (watched == m_appAreaWidget) {
         if (event->type() == QEvent::Resize)
             updateAppAreaSonWidgetSize();
@@ -584,8 +665,7 @@ bool MainPanelControl::eventFilter(QObject *watched, QEvent *event)
     }
 
     if (m_appDragWidget && watched == static_cast<QGraphicsView *>(m_appDragWidget)->viewport()) {
-        QDropEvent *e = static_cast<QDropEvent *>(event);
-        bool isContains = rect().contains(mapFromGlobal(m_appDragWidget->mapToGlobal(e->pos())));
+        bool isContains = rect().contains(mapFromGlobal(QCursor::pos()));
         if (isContains) {
             if (event->type() == QEvent::DragMove) {
                 handleDragMove(static_cast<QDragMoveEvent *>(event), true);
@@ -629,21 +709,19 @@ bool MainPanelControl::eventFilter(QObject *watched, QEvent *event)
     return QWidget::eventFilter(watched, event);
 }
 
+void MainPanelControl::enterEvent(QEvent *event)
+{
+    if (Utils::IS_WAYLAND_DISPLAY) {
+        Utils::updateCursor(this);
+    }
+
+    QWidget::enterEvent(event);
+}
+
 void MainPanelControl::mousePressEvent(QMouseEvent *e)
 {
     if (e->button() == Qt::LeftButton) {
         m_mousePressPos = e->globalPos();
-
-        QRect rect(m_desktopWidget->pos(), m_desktopWidget->size());
-        if (rect.contains(e->pos())) {
-            if (m_needRecoveryWin) {
-                // 手动点击 显示桌面窗口 后，鼠标移出时不再调用显/隐窗口进程，以手动点击设置为准
-                m_needRecoveryWin = false;
-            } else {
-                // 需求调整，鼠标移入，预览桌面时再点击显示桌面保持显示桌面状态，再点击才切换桌面显、隐状态
-                QProcess::startDetached("/usr/lib/deepin-daemon/desktop-toggle");
-            }
-        }
     }
 
     QWidget::mousePressEvent(e);
@@ -702,6 +780,15 @@ void MainPanelControl::startDrag(DockItem *dockItem)
         drag->setPixmap(pixmap);
         drag->setHotSpot(pixmap.rect().center() / pixmap.devicePixelRatioF());
     }
+
+    // isNeedBack 保存是否需要重置垃圾箱的AcceptDrops
+    // 设置垃圾箱插件AcceptDrops false
+    bool isNeedBack = false;
+    if (item->itemType() == DockItem::Plugins && m_trashItem && dockItem != m_trashItem) {
+        m_trashItem->centralWidget()->setAcceptDrops(false);
+        isNeedBack = true;
+    }
+
     drag->setMimeData(new QMimeData);
     drag->exec(Qt::MoveAction);
 
@@ -709,6 +796,10 @@ void MainPanelControl::startDrag(DockItem *dockItem)
         m_appDragWidget = nullptr;
         item->setDraging(false);
         item->update();
+
+        // isNeedBack是否需要设置垃圾箱插件AcceptDrops true
+        if (isNeedBack)
+            m_trashItem->centralWidget()->setAcceptDrops(true);
     }
 }
 
@@ -765,28 +856,7 @@ DockItem *MainPanelControl::dropTargetItem(DockItem *sourceItem, QPoint point)
 
     if (!targetItem && parentWidget == m_appAreaSonWidget) {
         // appitem调整顺序是，判断是否拖放在两边空白区域
-
-        if (!m_appAreaSonLayout->count())
-            return targetItem;
-
-        DockItem *first = qobject_cast<DockItem *>(m_appAreaSonLayout->itemAt(0)->widget());
-        DockItem *last = qobject_cast<DockItem *>(m_appAreaSonLayout->itemAt(m_appAreaSonLayout->count() - 1)->widget());
-
-        if (m_position == Dock::Top || m_position == Dock::Bottom) {
-
-            if (point.x() < 0) {
-                targetItem = first;
-            } else {
-                targetItem = last;
-            }
-        } else {
-
-            if (point.y() < 0) {
-                targetItem = first;
-            } else {
-                targetItem = last;
-            }
-        }
+        targetItem = sourceItem;
     }
 
     return targetItem;
@@ -799,6 +869,9 @@ void MainPanelControl::updateDisplayMode()
     resizeDesktopWidget();
 }
 
+/**把驻留应用和被打开的应用所在窗口移动到指定位置
+ * @brief MainPanelControl::moveAppSonWidget
+ */
 void MainPanelControl::moveAppSonWidget()
 {
     QRect rect(QPoint(0, 0), m_appAreaSonWidget->size());
@@ -841,6 +914,9 @@ void MainPanelControl::moveAppSonWidget()
     m_appAreaSonWidget->move(rect.x(), rect.y());
 }
 
+/**通知布局器，控件已发生变化，需要重新设置几何位置
+ * @brief MainPanelControl::updatePluginsLayout
+ */
 void MainPanelControl::updatePluginsLayout()
 {
     for (int i = 0; i < m_pluginLayout->count(); ++i) {
@@ -877,88 +953,65 @@ void MainPanelControl::paintEvent(QPaintEvent *event)
     painter.fillRect(m_fixedSpliter->geometry(), color);
     painter.fillRect(m_appSpliter->geometry(), color);
     painter.fillRect(m_traySpliter->geometry(), color);
-
-    //描绘桌面区域的颜色
-    painter.setOpacity(1);
-    QPen pen;
-    QColor penColor(0, 0, 0, 25);
-    pen.setWidth(2);
-    pen.setColor(penColor);
-    painter.setPen(pen);
-    painter.drawRect(m_desktopWidget->geometry());
-    if (m_isHover) {
-        painter.fillRect(m_desktopWidget->geometry(), QColor(255, 255, 255, 51));
-        return;
-    }
-    painter.fillRect(m_desktopWidget->geometry(), QColor(255, 255, 255, 25));
 }
 
+/**重新计算任务栏上应用图标、插件图标的大小，并设置
+ * @brief MainPanelControl::resizeDockIcon
+ */
 void MainPanelControl::resizeDockIcon()
 {
-    // 插件有点特殊，因为会引入第三方的插件，并不会受dock的缩放影响，我们只能限制我们自己的插件，否则会导致显示错误。
-    // 以下是受控制的插件
-    PluginsItem *trashPlugin = nullptr;
-    PluginsItem *shutdownPlugin = nullptr;
-    PluginsItem *keyboardPlugin = nullptr;
-    PluginsItem *notificationPlugin = nullptr;
+    // 总宽度
+    int totalLength = ((m_position == Position::Top) || (m_position == Position::Bottom)) ? width() : height();
 
-    //因为日期时间大小和其他插件大小有异，为了设置边距，在各插件中增加了一层布局
-    //因此需要通过多一层布局来获取各插件
+    // 减去托盘间隔区域
+    if (m_tray) {
+        totalLength -= (m_tray->trayVisibleItemCount() + 1) * 10;
+    }
+
+    // 减去3个分割线的宽度
+    totalLength -= 3 * SPLITER_SIZE;
+
+    // 减去显示桌面图标宽度
+    totalLength -= ((m_position == Position::Top) || (m_position == Position::Bottom)) ? m_desktopWidget->width() : m_desktopWidget->height();
+
+    int pluginItemCount = 0;
+    int calcPluginItemCount = 0;
+
+    // 因为日期时间大小和其他插件大小有异，为了设置边距，在各插件中增加了一层布局
+    // 因此需要通过多一层布局来获取各插件
     for (int i = 0; i < m_pluginLayout->count(); ++ i) {
         QLayout *layout = m_pluginLayout->itemAt(i)->layout();
         if (layout) {
-            PluginsItem *w = static_cast<PluginsItem *>(m_pluginLayout->itemAt(i)->widget());
+            PluginsItem *w = static_cast<PluginsItem *>(layout->itemAt(0)->widget());
             if (w) {
-                if (w->pluginName() == "trash") {
-                    trashPlugin = w;
-                } else if (w->pluginName() == "shutdown") {
-                    shutdownPlugin = w;
-                } else if (w->pluginName() == "onboard") {
-                    keyboardPlugin = w;
-                } else if (w->pluginName() == "notifications") {
-                    notificationPlugin = w;
+                // 如果插件大小由自己决定,则不参与计算需要减去其宽度,其他插件则需要参与计算并计数
+                if ((m_position == Position::Top || m_position == Position::Bottom) && (w->sizeHint().height() != -1)) {
+                    totalLength -= w->width();
+                } else if ((m_position == Position::Top || m_position == Position::Bottom) && (w->sizeHint().width() != -1)) {
+                    totalLength -= w->height();
+                } else {
+                    calcPluginItemCount ++;
                 }
+
+                // 所有插件个数,用于计算插件之间的间隔之和
+                pluginItemCount ++;
             }
         }
     }
 
-    // 总宽度
-    int totalLength = ((m_position == Position::Top) || (m_position == Position::Bottom)) ? width() : height();
-    // 减去托盘间隔区域
-    if (m_tray) {
-        totalLength -= (m_tray->trayVisableItemCount() + 1) * 10;
-    }
-    // 减去3个分割线的宽度
-    totalLength -= 3 * SPLITER_SIZE;
-
-    // 减去所有插件宽度，加上参与计算的4个插件宽度
-    if ((m_position == Position::Top) || (m_position == Position::Bottom)) {
-        totalLength -= m_pluginAreaWidget->width();
-        if (trashPlugin) totalLength += trashPlugin->width();
-        if (shutdownPlugin) totalLength += shutdownPlugin->width();
-        if (keyboardPlugin) totalLength += keyboardPlugin->width();
-        if (notificationPlugin) totalLength += notificationPlugin->width();
-        totalLength -= m_desktopWidget->width();
-    } else {
-        totalLength -= m_pluginAreaWidget->height();
-        if (trashPlugin) totalLength += trashPlugin->height();
-        if (shutdownPlugin) totalLength += shutdownPlugin->height();
-        if (keyboardPlugin) totalLength += keyboardPlugin->height();
-        if (notificationPlugin) totalLength += notificationPlugin->height();
-        totalLength -= m_desktopWidget->height();
-    }
+    // 减去插件间隔大小, 只有一个插件或没有插件都是间隔20,2个或以上每多一个插件多间隔10
+    if (pluginItemCount > 1)
+       totalLength -= (pluginItemCount + 1) * 10;
+    else
+       totalLength -= 20;
 
     if (totalLength < 0)
         return;
 
-    // 参与计算的插件的个数（包含托盘和插件，垃圾桶，关机，屏幕键盘）
-    int pluginCount = 0;
-    if (m_tray) {
-        pluginCount = m_tray->trayVisableItemCount() + (trashPlugin ? 1 : 0) + (shutdownPlugin ? 1 : 0) + (keyboardPlugin ? 1 : 0) + (notificationPlugin ? 1 : 0);
-    } else {
-        pluginCount = (trashPlugin ? 1 : 0) + (shutdownPlugin ? 1 : 0) + (keyboardPlugin ? 1 : 0) + (notificationPlugin ? 1 : 0);
-    }
-    // icon个数
+    // 参与计算的插件的个数包含托盘和插件
+    int pluginCount = m_tray ? m_tray->trayVisibleItemCount() + calcPluginItemCount : calcPluginItemCount;
+
+    // 需要计算的图标总数
     int iconCount = m_fixedAreaLayout->count() + m_appAreaSonLayout->count() + pluginCount;
     if (iconCount <= 0)
         return;
@@ -967,7 +1020,8 @@ void MainPanelControl::resizeDockIcon()
     int yu = (totalLength % iconCount);
     // icon宽度 = (总宽度-余数)/icon个数
     int iconSize = (totalLength - yu) / iconCount;
-    //计算插件图标的最大或最小值
+
+    // 计算插件图标的最大或最小值
     int tray_item_size = qBound(20, iconSize, 40);
     if ((m_position == Position::Top) || (m_position == Position::Bottom)) {
         tray_item_size = qMin(tray_item_size,height());
@@ -976,9 +1030,14 @@ void MainPanelControl::resizeDockIcon()
         tray_item_size = qMin(tray_item_size,width());
         tray_item_size = std::min(tray_item_size, width() - 20);
     }
-    //减去插件图标的大小后重新计算固定图标和应用图标的平均大小
+
+    if (tray_item_size < 20)
+        tray_item_size = 20;
+
+    // 减去插件图标的大小后重新计算固定图标和应用图标的平均大小
     totalLength -= tray_item_size * pluginCount;
     iconCount -= pluginCount;
+
     // 余数
     yu = (totalLength % iconCount);
     // icon宽度 = (总宽度-余数)/icon个数
@@ -986,20 +1045,20 @@ void MainPanelControl::resizeDockIcon()
 
     if ((m_position == Position::Top) || (m_position == Position::Bottom)) {
         if (iconSize >= height()) {
-            calcuDockIconSize(height(), height(), trashPlugin, shutdownPlugin, keyboardPlugin, notificationPlugin);
+            calcuDockIconSize(height(), height(), tray_item_size);
         } else {
-            calcuDockIconSize(iconSize, height(), trashPlugin, shutdownPlugin, keyboardPlugin, notificationPlugin);
+            calcuDockIconSize(iconSize, height(), tray_item_size);
         }
     } else {
         if (iconSize >= width()) {
-            calcuDockIconSize(width(), width(), trashPlugin, shutdownPlugin, keyboardPlugin, notificationPlugin);
+            calcuDockIconSize(width(), width(), tray_item_size);
         } else {
-            calcuDockIconSize(width(), iconSize, trashPlugin, shutdownPlugin, keyboardPlugin, notificationPlugin);
+            calcuDockIconSize(width(), iconSize, tray_item_size);
         }
     }
 }
 
-void MainPanelControl::calcuDockIconSize(int w, int h, PluginsItem *trashPlugin, PluginsItem *shutdownPlugin, PluginsItem *keyboardPlugin, PluginsItem *notificationPlugin)
+void MainPanelControl::calcuDockIconSize(int w, int h, int traySize)
 {
     int appItemSize = qMin(w, h);
 
@@ -1021,22 +1080,8 @@ void MainPanelControl::calcuDockIconSize(int w, int h, PluginsItem *trashPlugin,
         m_appAreaSonLayout->itemAt(i)->widget()->setFixedSize(appItemSize, appItemSize);
     }
 
-    // 托盘上每个图标大小
-    int tray_item_size = 20;
-
-    if ((m_position == Position::Top) || (m_position == Position::Bottom)) {
-        w = qBound(20, w, 40);
-        tray_item_size = std::min(w, h - 20);
-    } else {
-        h = qBound(20, h, 40);
-        tray_item_size = std::min(w - 20, h);
-    }
-
-    if (tray_item_size < 20)
-        return;
-
     if (m_tray) {
-        m_tray->centralWidget()->setProperty("iconSize", tray_item_size);
+        m_tray->centralWidget()->setProperty("iconSize", traySize);
     }
 
     //因为日期时间大小和其他插件大小有异，为了设置边距，在各插件中增加了一层布局
@@ -1045,11 +1090,11 @@ void MainPanelControl::calcuDockIconSize(int w, int h, PluginsItem *trashPlugin,
         // 三方插件
         for (int i = 0; i < m_pluginLayout->count(); ++ i) {
             QLayout *layout = m_pluginLayout->itemAt(i)->layout();
-            if (layout) {
+            if (layout && layout->itemAt(0)) {
                 PluginsItem *pItem = static_cast<PluginsItem *>(layout->itemAt(0)->widget());
                 if (pItem) {
                     if (pItem->sizeHint().height() == -1) {
-                        pItem->setFixedSize(tray_item_size, tray_item_size);
+                        pItem->setFixedSize(traySize, traySize);
                     } else if (pItem->sizeHint().height() > height()) {
                         pItem->resize(pItem->width(), height());
                     }
@@ -1064,7 +1109,7 @@ void MainPanelControl::calcuDockIconSize(int w, int h, PluginsItem *trashPlugin,
                 PluginsItem *pItem = static_cast<PluginsItem *>(layout->itemAt(0)->widget());
                 if (pItem) {
                     if (pItem->sizeHint().width() == -1) {
-                        pItem->setFixedSize(tray_item_size, tray_item_size);
+                        pItem->setFixedSize(traySize, traySize);
                     } else if (pItem->sizeHint().width() > width()) {
                         pItem->resize(width(), pItem->height());
                     }
@@ -1081,10 +1126,10 @@ void MainPanelControl::calcuDockIconSize(int w, int h, PluginsItem *trashPlugin,
 
     if ((m_position == Position::Top) || (m_position == Position::Bottom)) {
         appTopAndBottomMargin = (m_fixedAreaWidget->height() - appItemSize) / 2;
-        trayTopAndBottomMargin = (m_trayAreaWidget->height() - tray_item_size) / 2;
+        trayTopAndBottomMargin = (m_trayAreaWidget->height() - traySize) / 2;
     } else {
         appLeftAndRightMargin = (m_fixedAreaWidget->width() - appItemSize) / 2;
-        trayLeftAndRightMargin = (m_trayAreaWidget->width() - tray_item_size) / 2;
+        trayLeftAndRightMargin = (m_trayAreaWidget->width() - traySize) / 2;
     }
 
     m_fixedAreaLayout->setContentsMargins(appLeftAndRightMargin, appTopAndBottomMargin, appLeftAndRightMargin, appTopAndBottomMargin);
@@ -1095,7 +1140,7 @@ void MainPanelControl::calcuDockIconSize(int w, int h, PluginsItem *trashPlugin,
     //而不对日期时间插件设置边距
     for (int i = 0; i < m_pluginLayout->count(); ++ i) {
         QLayout *layout = m_pluginLayout->itemAt(i)->layout();
-        if (layout) {
+        if (layout && layout->itemAt(0)) {
             PluginsItem *pItem = static_cast<PluginsItem *>(layout->itemAt(0)->widget());
 
             if (pItem && pItem->pluginName() != "datetime") {
@@ -1105,18 +1150,9 @@ void MainPanelControl::calcuDockIconSize(int w, int h, PluginsItem *trashPlugin,
     }
 }
 
-void MainPanelControl::getTrayVisableItemCount()
-{
-    if (m_trayAreaLayout->count() > 0) {
-        TrayPluginItem *w = static_cast<TrayPluginItem *>(m_trayAreaLayout->itemAt(0)->widget());
-        m_trayIconCount = w->trayVisableItemCount();
-    } else {
-        m_trayIconCount = 0;
-    }
-
-    resizeDockIcon();
-}
-
+/**时尚模式没有‘显示桌面’区域
+ * @brief MainPanelControl::resizeDesktopWidget
+ */
 void MainPanelControl::resizeDesktopWidget()
 {
     if (m_position == Position::Right || m_position == Position::Left)
@@ -1145,3 +1181,13 @@ bool MainPanelControl::checkNeedShowDesktop()
     qDebug() << "wm call GetIsShowDesktop fail, res:" << reply.type();
     return false;
 }
+
+/**
+ * @brief MainWindow::appIsOnDock 判断指定的应用（驻留和运行显示在任务栏的所有应用）是否在任务栏上
+ * @param appDesktop 应用的desktop文件的完整路径
+ * @return true: 在任务栏；false: 不在任务栏
+ */
+bool MainPanelControl::appIsOnDock(const QString &appDesktop)
+{
+    return DockItemManager::instance()->appIsOnDock(appDesktop);
+}
