@@ -23,7 +23,6 @@
 #include "mainwindow.h"
 #include "mainpanelcontrol.h"
 #include "dockitemmanager.h"
-#include "utils.h"
 #include "menuworker.h"
 
 #include <DStyle>
@@ -38,6 +37,7 @@
 #include <QScreen>
 #include <QGuiApplication>
 #include <QX11Info>
+#include <QtConcurrent>
 #include <qpa/qplatformwindow.h>
 
 #include <X11/X.h>
@@ -52,11 +52,12 @@
 #define MAINWINDOW_MIN_SIZE       (40)
 #define DRAG_AREA_SIZE (5)
 
+#define DRAG_STATE_PROP "DRAG_STATE"
+
 using org::kde::StatusNotifierWatcher;
-using DBusDock = com::deepin::dde::daemon::Dock;
 
 // let startdde know that we've already started.
-void RegisterDdeSession()
+void MainWindow::RegisterDdeSession()
 {
     QString envName("DDE_SESSION_PROCESS_COOKIE_ID");
 
@@ -88,19 +89,26 @@ MainWindow::MainWindow(QWidget *parent)
     , m_sniWatcher(new StatusNotifierWatcher(SNI_WATCHER_SERVICE, SNI_WATCHER_PATH, QDBusConnection::sessionBus(), this))
     , m_dragWidget(new DragWidget(this))
     , m_launched(false)
+    , m_updateDragAreaTimer(new QTimer(this))
 {
     setAttribute(Qt::WA_TranslucentBackground);
     setAttribute(Qt::WA_X11DoNotAcceptFocus);
 
+    Qt::WindowFlags flags = Qt::WindowStaysOnTopHint | Qt::FramelessWindowHint | Qt::Window;
     //1 确保这两行代码的先后顺序，否则会导致任务栏界面不再置顶
-    setWindowFlags(Qt::WindowDoesNotAcceptFocus);
+    setWindowFlags(windowFlags() | flags | Qt::WindowDoesNotAcceptFocus);
 
-    const auto display = QX11Info::display();
-    if (!display) {
-        qWarning() << "QX11Info::display() is " << display;
-    } else {
-        //2 确保这两行代码的先后顺序，否则会导致任务栏界面不再置顶
-        XcbMisc::instance()->set_window_type(xcb_window_t(this->winId()), XcbMisc::Dock);
+    if (Utils::IS_WAYLAND_DISPLAY)
+        setWindowFlag(Qt::FramelessWindowHint, false); // 会导致设置圆角为0时无效
+
+    if (DGuiApplicationHelper::isXWindowPlatform()) {
+        const auto display = QX11Info::display();
+        if (!display) {
+            qWarning() << "QX11Info::display() is " << display;
+        } else {
+            //2 确保这两行代码的先后顺序，否则会导致任务栏界面不再置顶
+            XcbMisc::instance()->set_window_type(xcb_window_t(this->winId()), XcbMisc::Dock);
+        }
     }
 
     setMouseTracking(true);
@@ -109,49 +117,55 @@ MainWindow::MainWindow(QWidget *parent)
     DPlatformWindowHandle::enableDXcbForWindow(this, true);
     m_platformWindowHandle.setEnableBlurWindow(true);
     m_platformWindowHandle.setTranslucentBackground(true);
-    m_platformWindowHandle.setWindowRadius(0);
     m_platformWindowHandle.setShadowOffset(QPoint(0, 5));
     m_platformWindowHandle.setShadowColor(QColor(0, 0, 0, 0.3 * 255));
 
     m_mainPanel->setDisplayMode(m_multiScreenWorker->displayMode());
 
+    initMember();
     initSNIHost();
     initComponents();
     initConnections();
 
     resetDragWindow();
 
-    m_mainPanel->setDelegate(this);
     for (auto item : DockItemManager::instance()->itemList())
         m_mainPanel->insertItem(-1, item);
 
     m_dragWidget->setMouseTracking(true);
     m_dragWidget->setFocusPolicy(Qt::NoFocus);
 
-    if ((Top == m_multiScreenWorker->position()) || (Bottom == m_multiScreenWorker->position())) {
-        m_dragWidget->setCursor(Qt::SizeVerCursor);
-    } else {
-        m_dragWidget->setCursor(Qt::SizeHorCursor);
+    if (!Utils::IS_WAYLAND_DISPLAY) {
+        if ((Top == m_multiScreenWorker->position()) || (Bottom == m_multiScreenWorker->position())) {
+            m_dragWidget->setCursor(Qt::SizeVerCursor);
+        } else {
+            m_dragWidget->setCursor(Qt::SizeHorCursor);
+        }
     }
 }
 
-MainWindow::~MainWindow()
-{
-
-}
-
+/**
+ * @brief MainWindow::launch
+ * 任务栏初次启动时调用此方法，里面是做了一些初始化操作
+ */
 void MainWindow::launch()
 {
     if (!qApp->property("CANSHOW").toBool())
         return;
 
     m_launched = true;
-    qApp->processEvents();
-    setVisible(true);
     m_multiScreenWorker->initShow();
     m_shadowMaskOptimizeTimer->start();
+    QTimer::singleShot(0, this, [ this ] { this->setVisible(true); });
 }
 
+/**
+ * @brief MainWindow::callShow
+ * 此方法是被外部进程通过DBus调用的。
+ * @note 当任务栏以-r参数启动时，其不会显示界面，需要在外部通过DBus调用此接口之后才会显示界面，
+ * 这里是以前为了优化任务栏的启动速度做的处理，当任务栏启动时，此时窗管进程可能还未启动完全，
+ * 部分设置未初始化完等，导致任务栏显示的界面异常，所以留下此接口，被startdde延后调用
+ */
 void MainWindow::callShow()
 {
     static bool flag = false;
@@ -165,12 +179,13 @@ void MainWindow::callShow()
     launch();
 
     // 预留200ms提供给窗口初始化再通知startdde，不影响启动速度
-    QTimer::singleShot(200, this, []{
-        qDebug() << "\n\ndde-dock startup RegisterDdeSession";
-        RegisterDdeSession();
-    });
+    QTimer::singleShot(200, this, &MainWindow::RegisterDdeSession);
 }
 
+/**
+ * @brief MainWindow::relaodPlugins
+ * 需要重新加载插件时，此接口会被调用，目前是用于任务栏的安全模式退出时调用
+ */
 void MainWindow::relaodPlugins()
 {
     if (qApp->property("PLUGINSLOADED").toBool()) {
@@ -181,35 +196,19 @@ void MainWindow::relaodPlugins()
     qApp->setProperty("PLUGINSLOADED", true);
 }
 
-void MainWindow::showEvent(QShowEvent *e)
-{
-    QWidget::showEvent(e);
-
-    //    connect(qGuiApp, &QGuiApplication::primaryScreenChanged,
-    //    windowHandle(), [this](QScreen * new_screen) {
-    //        QScreen *old_screen = windowHandle()->screen();
-    //        windowHandle()->setScreen(new_screen);
-    //        // 屏幕变化后可能导致控件缩放比变化，此时应该重设控件位置大小
-    //        // 比如：窗口大小为 100 x 100, 显示在缩放比为 1.0 的屏幕上，此时窗口的真实大小 = 100x100
-    //        // 随后窗口被移动到了缩放比为 2.0 的屏幕上，应该将真实大小改为 200x200。另外，只能使用
-    //        // QPlatformWindow直接设置大小来绕过QWidget和QWindow对新旧geometry的比较。
-    //        const qreal scale = devicePixelRatioF();
-    //        const QPoint screenPos = new_screen->geometry().topLeft();
-    //        const QPoint posInScreen = this->pos() - old_screen->geometry().topLeft();
-    //        const QPoint pos = screenPos + posInScreen * scale;
-    //        const QSize size = this->size() * scale;
-
-    //        windowHandle()->handle()->setGeometry(QRect(pos, size));
-    //    }, Qt::UniqueConnection);
-
-    //    windowHandle()->setScreen(qGuiApp->primaryScreen());
-}
-
+/**
+ * @brief MainWindow::mousePressEvent
+ * @param e
+ * @note 右键显示任务栏的菜单
+ */
 void MainWindow::mousePressEvent(QMouseEvent *e)
 {
     e->ignore();
     if (e->button() == Qt::RightButton) {
-        m_menuWorker->showDockSettingsMenu();
+        QTimer::singleShot(10, this, [this]{
+            QMenu *menu = new QMenu(this);
+            m_menuWorker->showDockSettingsMenu(menu);
+        });
         return;
     }
 }
@@ -241,10 +240,16 @@ void MainWindow::mouseMoveEvent(QMouseEvent *e)
 void MainWindow::moveEvent(QMoveEvent *event)
 {
     Q_UNUSED(event);
+
+    if (!qApp->property(DRAG_STATE_PROP).toBool())
+        m_updateDragAreaTimer->start();
 }
 
 void MainWindow::resizeEvent(QResizeEvent *event)
 {
+    if (!qApp->property(DRAG_STATE_PROP).toBool())
+        m_updateDragAreaTimer->start();
+
     // 任务栏大小、位置、模式改变都会触发resize，发射大小改变信号，供依赖项目更新位置
     Q_EMIT panelGeometryChanged();
 
@@ -259,6 +264,17 @@ void MainWindow::dragEnterEvent(QDragEnterEvent *e)
     QWidget::dragEnterEvent(e);
 }
 
+void MainWindow::initMember()
+{
+    //INFO 这里要大于动画的300ms，否则可能动画过程中这个定时器就被触发了
+    m_updateDragAreaTimer->setInterval(500);
+    m_updateDragAreaTimer->setSingleShot(true);
+}
+
+/**
+ * @brief MainWindow::initSNIHost
+ * @note 将Dock注册到StatusNotifierWatcher服务上
+ */
 void MainWindow::initSNIHost()
 {
     // registor dock as SNI Host on dbus
@@ -305,16 +321,22 @@ void MainWindow::initConnections()
     connect(DockItemManager::instance(), &DockItemManager::itemInserted, m_mainPanel, &MainPanelControl::insertItem, Qt::DirectConnection);
     connect(DockItemManager::instance(), &DockItemManager::itemRemoved, m_mainPanel, &MainPanelControl::removeItem, Qt::DirectConnection);
     connect(DockItemManager::instance(), &DockItemManager::itemUpdated, m_mainPanel, &MainPanelControl::itemUpdated, Qt::DirectConnection);
-    connect(DockItemManager::instance(), &DockItemManager::trayVisableCountChanged, this, &MainWindow::getTrayVisableItemCount, Qt::QueuedConnection);
+    connect(DockItemManager::instance(), &DockItemManager::trayVisableCountChanged, this, &MainWindow::resizeDockIcon, Qt::QueuedConnection);
     connect(DockItemManager::instance(), &DockItemManager::requestWindowAutoHide, m_menuWorker, &MenuWorker::setAutoHide);
     connect(m_mainPanel, &MainPanelControl::itemMoved, DockItemManager::instance(), &DockItemManager::itemMoved, Qt::DirectConnection);
     connect(m_mainPanel, &MainPanelControl::itemAdded, DockItemManager::instance(), &DockItemManager::itemAdded, Qt::DirectConnection);
 
-    connect(m_dragWidget, &DragWidget::dragPointOffset, m_multiScreenWorker, [ = ] {m_multiScreenWorker->onDragStateChanged(true);});
-    connect(m_dragWidget, &DragWidget::dragFinished, m_multiScreenWorker, [ = ] {m_multiScreenWorker->onDragStateChanged(false);});
+    // -拖拽任务栏改变高度或宽度-------------------------------------------------------------------------------
+    connect(m_updateDragAreaTimer, &QTimer::timeout, this, &MainWindow::resetDragWindow);
+    //TODO 后端考虑删除这块，目前还不能删除，调整任务栏高度的时候，任务栏外部区域有变化
+    connect(m_updateDragAreaTimer, &QTimer::timeout, m_multiScreenWorker, &MultiScreenWorker::onRequestUpdateRegionMonitor);
+
+    connect(m_dragWidget, &DragWidget::dragPointOffset, this, [ = ] { qApp->setProperty(DRAG_STATE_PROP, true); });
+    connect(m_dragWidget, &DragWidget::dragFinished, this, [ = ] { qApp->setProperty(DRAG_STATE_PROP, false); });
 
     connect(m_dragWidget, &DragWidget::dragPointOffset, this, &MainWindow::onMainWindowSizeChanged);
-    connect(m_dragWidget, &DragWidget::dragFinished, this, &MainWindow::onDragFinished);
+    connect(m_dragWidget, &DragWidget::dragFinished, this, &MainWindow::resetDragWindow);   //　更新拖拽区域
+    // ----------------------------------------------------------------------------------------------------
 
     connect(DGuiApplicationHelper::instance(), &DGuiApplicationHelper::themeTypeChanged, this, &MainWindow::themeTypeChanged);
 
@@ -325,29 +347,23 @@ void MainWindow::initConnections()
 
     connect(m_multiScreenWorker, &MultiScreenWorker::requestUpdateDockEntry, DockItemManager::instance(), &DockItemManager::requestUpdateDockItem);
 
-    //　更新拖拽区域
-    connect(m_multiScreenWorker, &MultiScreenWorker::requestUpdateDragArea, this, &MainWindow::resetDragWindow);
-
     // 响应后端触控屏拖拽任务栏高度长按信号
     connect(TouchSignalManager::instance(), &TouchSignalManager::middleTouchPress, this, &MainWindow::touchRequestResizeDock);
-    connect(TouchSignalManager::instance(), &TouchSignalManager::touchMove, m_dragWidget, [ this ]() {
-        static QPoint lastPos;
-        QPoint curPos = QCursor::pos();
-        if (lastPos == curPos) {
-            return;
-        }
-        lastPos = curPos;
-        qApp->postEvent(m_dragWidget, new QMouseEvent(QEvent::MouseMove, m_dragWidget->mapFromGlobal(curPos)
-                                                      , QPoint(), curPos, Qt::LeftButton, Qt::LeftButton
-                                                      , Qt::NoModifier, Qt::MouseEventSynthesizedByApplication));
-    });
+    connect(TouchSignalManager::instance(), &TouchSignalManager::touchMove, m_dragWidget, &DragWidget::onTouchMove);
 }
 
-void MainWindow::getTrayVisableItemCount()
+/**
+ * @brief MainWindow::getTrayVisableItemCount
+ * 重新获取以下当前托盘区域有多少个可见的图标，并更新图标的大小
+ */
+void MainWindow::resizeDockIcon()
 {
-    m_mainPanel->getTrayVisableItemCount();
+    m_mainPanel->resizeDockIcon();
 }
 
+/**
+ * @brief MainWindow::adjustShadowMask 更新任务栏的圆角大小（时尚模式下才有圆角效果）
+ */
 void MainWindow::adjustShadowMask()
 {
     if (!m_launched || m_shadowMaskOptimizeTimer->isActive())
@@ -378,6 +394,10 @@ void MainWindow::onDbusNameOwnerChanged(const QString &name, const QString &oldO
     }
 }
 
+/**
+ * @brief MainWindow::setEffectEnabled
+ * @param enabled 根据当前系统是否enabled特效来更新任务栏的外观样式
+ */
 void MainWindow::setEffectEnabled(const bool enabled)
 {
     setMaskColor(AutoColor);
@@ -387,16 +407,19 @@ void MainWindow::setEffectEnabled(const bool enabled)
     m_platformWindowHandle.setBorderWidth(enabled ? 1 : 0);
 }
 
+/**
+ * @brief MainWindow::setComposite
+ * @param hasComposite 系统是否支持混成（也就是特效）
+ */
 void MainWindow::setComposite(const bool hasComposite)
 {
     setEffectEnabled(hasComposite);
 }
 
-bool MainWindow::appIsOnDock(const QString &appDesktop)
-{
-    return DockItemManager::instance()->appIsOnDock(appDesktop);
-}
-
+/**
+ * @brief MainWindow::resetDragWindow 更新任务栏的拖拽区域
+ * @note 任务栏远离屏幕的一边是支持拖拽的，由一个不可见的widget提拽支持，当任务栏的geometry发生变化的时候，此拖拽区域也需要更新其自身的geometry
+ */
 void MainWindow::resetDragWindow()
 {
     switch (m_multiScreenWorker->position()) {
@@ -449,6 +472,56 @@ void MainWindow::resetDragWindow()
     }
 }
 
+void MainWindow::resizeDock(int offset, bool dragging)
+{
+    qApp->setProperty(DRAG_STATE_PROP, dragging);
+
+    const QRect &rect = m_multiScreenWorker->getDockShowMinGeometry(m_multiScreenWorker->deskScreen());
+    QRect newRect;
+    switch (m_multiScreenWorker->position()) {
+    case Top: {
+       newRect.setX(rect.x());
+       newRect.setY(rect.y());
+       newRect.setWidth(rect.width());
+       newRect.setHeight(qBound(MAINWINDOW_MIN_SIZE, offset, MAINWINDOW_MAX_SIZE));
+   }
+        break;
+    case Bottom: {
+        newRect.setX(rect.x());
+        newRect.setY(rect.y() + rect.height() - qBound(MAINWINDOW_MIN_SIZE, offset, MAINWINDOW_MAX_SIZE));
+        newRect.setWidth(rect.width());
+        newRect.setHeight(qBound(MAINWINDOW_MIN_SIZE, offset, MAINWINDOW_MAX_SIZE));
+    }
+        break;
+    case Left: {
+        newRect.setX(rect.x());
+        newRect.setY(rect.y());
+        newRect.setWidth(qBound(MAINWINDOW_MIN_SIZE, offset, MAINWINDOW_MAX_SIZE));
+        newRect.setHeight(rect.height());
+    }
+        break;
+    case Right: {
+        newRect.setX(rect.x() + rect.width() - qBound(MAINWINDOW_MIN_SIZE, offset, MAINWINDOW_MAX_SIZE));
+        newRect.setY(rect.y());
+        newRect.setWidth(qBound(MAINWINDOW_MIN_SIZE, offset, MAINWINDOW_MAX_SIZE));
+        newRect.setHeight(rect.height());
+    }
+        break;
+    }
+
+    // 更新界面大小
+    m_mainPanel->setFixedSize(newRect.size());
+    setFixedSize(newRect.size());
+    move(newRect.topLeft());
+
+    if (!dragging)
+        resetDragWindow();
+}
+
+/**
+ * @brief MainWindow::onMainWindowSizeChanged 任务栏拖拽过程中会不听调用此方法更新自身大小
+ * @param offset 拖拽时的坐标偏移量
+ */
 void MainWindow::onMainWindowSizeChanged(QPoint offset)
 {
     const QRect &rect = m_multiScreenWorker->dockRect(m_multiScreenWorker->deskScreen()
@@ -463,28 +536,28 @@ void MainWindow::onMainWindowSizeChanged(QPoint offset)
         newRect.setWidth(rect.width());
         newRect.setHeight(qBound(MAINWINDOW_MIN_SIZE, rect.height() + offset.y(), MAINWINDOW_MAX_SIZE));
     }
-    break;
+        break;
     case Bottom: {
         newRect.setX(rect.x());
         newRect.setY(rect.y() + rect.height() - qBound(MAINWINDOW_MIN_SIZE, rect.height() - offset.y(), MAINWINDOW_MAX_SIZE));
         newRect.setWidth(rect.width());
         newRect.setHeight(qBound(MAINWINDOW_MIN_SIZE, rect.height() - offset.y(), MAINWINDOW_MAX_SIZE));
     }
-    break;
+        break;
     case Left: {
         newRect.setX(rect.x());
         newRect.setY(rect.y());
         newRect.setWidth(qBound(MAINWINDOW_MIN_SIZE, rect.width() + offset.x(), MAINWINDOW_MAX_SIZE));
         newRect.setHeight(rect.height());
     }
-    break;
+        break;
     case Right: {
         newRect.setX(rect.x() + rect.width() - qBound(MAINWINDOW_MIN_SIZE, rect.width() - offset.x(), MAINWINDOW_MAX_SIZE));
         newRect.setY(rect.y());
         newRect.setWidth(qBound(MAINWINDOW_MIN_SIZE, rect.width() - offset.x(), MAINWINDOW_MAX_SIZE));
         newRect.setHeight(rect.height());
     }
-    break;
+        break;
     }
 
     // 更新界面大小
@@ -493,12 +566,10 @@ void MainWindow::onMainWindowSizeChanged(QPoint offset)
     move(newRect.topLeft());
 }
 
-void MainWindow::onDragFinished()
-{
-    qDebug() << "drag finished";
-    resetDragWindow();
-}
-
+/**
+ * @brief MainWindow::themeTypeChanged 系统主题发生变化时，此方法被调用
+ * @param themeType 当前系统主题
+ */
 void MainWindow::themeTypeChanged(DGuiApplicationHelper::ColorType themeType)
 {
     if (m_wmHelper->hasComposite()) {
@@ -509,6 +580,9 @@ void MainWindow::themeTypeChanged(DGuiApplicationHelper::ColorType themeType)
     }
 }
 
+/**
+ * @brief MainWindow::touchRequestResizeDock 触屏情况用手指调整任务栏高度或宽度
+ */
 void MainWindow::touchRequestResizeDock()
 {
     const QPoint touchPos(QCursor::pos());
@@ -549,6 +623,10 @@ void MainWindow::touchRequestResizeDock()
                                                   , Qt::NoModifier, Qt::MouseEventSynthesizedByApplication));
 }
 
+/**
+ * @brief MainWindow::setGeometry
+ * @param rect 设置任务栏的位置和大小，重写此函数时为了及时发出panelGeometryChanged信号，最终供外部DBus调用方使用
+ */
 void MainWindow::setGeometry(const QRect &rect)
 {
     if (rect == this->geometry()) {
@@ -567,22 +645,21 @@ void MainWindow::sendNotifications()
     actionButton << "reload" << tr("Exit Safe Mode");
     QVariantMap hints;
     hints["x-deepin-action-reload"] = QString("dbus-send,--session,--dest=com.deepin.dde.Dock,--print-reply,/com/deepin/dde/Dock,com.deepin.dde.Dock.ReloadPlugins");
-    QTimer::singleShot(0, this, [=] {
+    // 在进入安全模式时，执行此DBUS耗时25S左右，导致任务栏显示阻塞，所以使用线程调用
+    QtConcurrent::run(QThreadPool::globalInstance(), [=] {
         DDBusSender()
-            .service("com.deepin.dde.Notification")
-            .path("/com/deepin/dde/Notification")
-            .interface("com.deepin.dde.Notification")
-            .method(QString("Notify"))
-            .arg(QString("dde-control-center"))                                            // appname
-            .arg(static_cast<uint>(0))                                                     // id
-            .arg(QString("preferences-system"))                                            // icon
-            .arg(QString(tr("Dock - Safe Mode")))                                          // summary
-            .arg(QString(tr("The Dock is in safe mode, please exit to show it properly"))) // content
-            .arg(actionButton)                                                             // actions
-            .arg(hints)                                                                    // hints
-            .arg(15000)                                                                    // timeout
-            .call();
+                .service("com.deepin.dde.Notification")
+                .path("/com/deepin/dde/Notification")
+                .interface("com.deepin.dde.Notification")
+                .method(QString("Notify"))
+                .arg(QString("dde-control-center"))                                            // appname
+                .arg(static_cast<uint>(0))                                                     // id
+                .arg(QString("preferences-system"))                                            // icon
+                .arg(QString(tr("Dock - Safe Mode")))                                          // summary
+                .arg(QString(tr("The Dock is in safe mode, please exit to show it properly"))) // content
+                .arg(actionButton)                                                             // actions
+                .arg(hints)                                                                    // hints
+                .arg(15000)                                                                    // timeout
+                .call();
     });
 }
-
-#include "mainwindow.moc"
