@@ -20,8 +20,7 @@
  */
 
 #include "soundapplet.h"
-#include "sinkinputwidget.h"
-#include "componments/horizontalseparator.h"
+#include "util/horizontalseperator.h"
 #include "../widgets/tipswidget.h"
 #include "../frame/util/imageutil.h"
 #include "util/utils.h"
@@ -30,18 +29,23 @@
 #include <DApplication>
 #include <DStandardItem>
 #include <DFontSizeManager>
+#include <DApplicationHelper>
 
 #include <QLabel>
 #include <QIcon>
 #include <QScrollBar>
+#include <QPainter>
+#include <QListIterator>
 
+#define SEPARATOR_HEIGHT 2
 #define WIDTH       260
 #define MAX_HEIGHT  300
 #define ICON_SIZE   24
 #define ITEM_HEIGHT 24
 #define ITEM_SPACING 5
 #define DEVICE_SPACING 10
-#define SLIDER_HIGHT 32
+#define SLIDER_HIGHT 70
+#define TITLE_HEIGHT 46
 #define GSETTING_SOUND_OUTPUT_SLIDER "soundOutputSlider"
 
 DWIDGET_USE_NAMESPACE
@@ -108,13 +112,14 @@ void Port::setCardId(const uint &cardId)
 
 SoundApplet::SoundApplet(QWidget *parent)
     : QScrollArea(parent)
-    , m_centralWidget(new QWidget)
-    , m_volumeBtn(new DIconButton(this))
-    , m_volumeIconMax(new QLabel)
-    , m_volumeSlider(new VolumeSlider)
-    , m_soundShow(new TipsWidget)
-    , m_separator(new HorizontalSeparator)
-    , m_deviceLabel(nullptr)
+    , m_centralWidget(new QWidget(this))
+    , m_volumeIconMin(new QLabel(this))
+    , m_volumeIconMax(new QLabel(this))
+    , m_volumeSlider(new VolumeSlider(this))
+    , m_soundShow(new QLabel(this))
+    , m_deviceLabel(new QLabel(this))
+    , m_seperator(new HorizontalSeperator(this))
+    , m_secondSeperator(new HorizontalSeperator(this))
     , m_audioInter(new DBusAudio("com.deepin.daemon.Audio", "/com/deepin/daemon/Audio", QDBusConnection::sessionBus(), this))
     , m_defSinkInter(nullptr)
     , m_listView(new DListView(this))
@@ -122,103 +127,112 @@ SoundApplet::SoundApplet(QWidget *parent)
     , m_deviceInfo("")
     , m_lastPort(nullptr)
     , m_gsettings(Utils::ModuleSettingsPtr("sound", QByteArray(), this))
-
 {
     initUi();
+
+    m_volumeIconMin->installEventFilter(this);
 }
 
 void SoundApplet::initUi()
 {
+    m_listView->setFrameShape(QFrame::NoFrame);
     m_listView->setEditTriggers(DListView::NoEditTriggers);
     m_listView->setSelectionMode(QAbstractItemView::NoSelection);
     m_listView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
     m_listView->setBackgroundType(DStyledItemDelegate::NoBackground);
+    m_listView->setItemRadius(0);
     m_listView->setSizeAdjustPolicy(QAbstractScrollArea::AdjustToContents);
-    m_listView->setViewportMargins(QMargins(0, 0, 12, 0));
-    m_listView->setSpacing(ITEM_SPACING);
     m_listView->setFixedHeight(0);
+    m_listView->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
+    m_listView->setModel(m_model);
 
     m_centralWidget->setAccessibleName("volumn-centralwidget");
-    m_volumeBtn->setAccessibleName("volume-button");
+    m_volumeIconMin->setAccessibleName("volume-button");
     m_volumeIconMax->setAccessibleName("volume-iconmax");
     m_volumeSlider->setAccessibleName("volume-slider");
     m_soundShow->setAccessibleName("volume-soundtips");
-    this->horizontalScrollBar()->setAccessibleName("volume-horizontalscrollbar");
-    this->verticalScrollBar()->setAccessibleName("volume-verticalscrollbar");
+    horizontalScrollBar()->setAccessibleName("volume-horizontalscrollbar");
+    verticalScrollBar()->setAccessibleName("volume-verticalscrollbar");
 
+    m_volumeIconMin->setFixedSize(ICON_SIZE, ICON_SIZE);
     m_volumeIconMax->setFixedSize(ICON_SIZE, ICON_SIZE);
 
     m_soundShow->setText(QString("%1%").arg(0));
+    m_soundShow->setFixedHeight(TITLE_HEIGHT);
+    m_soundShow->setForegroundRole(QPalette::BrightText);
+    DFontSizeManager::instance()->bind(m_soundShow, DFontSizeManager::T8, QFont::Medium);
 
-    m_deviceLabel = new TipsWidget;
     m_deviceLabel->setText(tr("Device"));
+    m_deviceLabel->setFixedHeight(TITLE_HEIGHT);
+    m_deviceLabel->setForegroundRole(QPalette::BrightText);
     DFontSizeManager::instance()->bind(m_deviceLabel, DFontSizeManager::T4, QFont::Medium);
 
-    QHBoxLayout *deviceLayout = new QHBoxLayout;
-    deviceLayout->addSpacing(2);
-    deviceLayout->addWidget(m_deviceLabel, 0, Qt::AlignLeft);
-    deviceLayout->addWidget(m_soundShow, 0, Qt::AlignRight);
-    deviceLayout->setSpacing(0);
-    deviceLayout->setMargin(0);
-
-    QVBoxLayout *deviceLineLayout = new QVBoxLayout;
-    deviceLineLayout->addLayout(deviceLayout);
-    deviceLineLayout->addWidget(m_separator);
-    deviceLineLayout->setMargin(0);
-    deviceLineLayout->setSpacing(DEVICE_SPACING);
-
-    QHBoxLayout *volumeCtrlLayout = new QHBoxLayout;
-    volumeCtrlLayout->addSpacing(2);
-    volumeCtrlLayout->addWidget(m_volumeBtn);
-    volumeCtrlLayout->addSpacing(10);
-    volumeCtrlLayout->addWidget(m_volumeSlider);
-    volumeCtrlLayout->addSpacing(10);
-    volumeCtrlLayout->addWidget(m_volumeIconMax);
-    volumeCtrlLayout->setSpacing(0);
-    volumeCtrlLayout->setMargin(0);
-
-    m_volumeBtn->setFixedSize(ICON_SIZE, ICON_SIZE);
-    m_volumeBtn->setIconSize(QSize(ICON_SIZE, ICON_SIZE));
-    m_volumeBtn->setFlat(true);
-
     m_volumeSlider->setFixedHeight(SLIDER_HIGHT);
     m_volumeSlider->setMinimum(0);
     m_volumeSlider->setMaximum(m_audioInter->maxUIVolume() * 100.0f);
+
+    // 标题部分
+    QHBoxLayout *deviceLayout = new QHBoxLayout;
+    deviceLayout->setSpacing(0);
+    deviceLayout->setMargin(0);
+    deviceLayout->setContentsMargins(20, 0, 10, 0);
+    deviceLayout->addWidget(m_deviceLabel, 0, Qt::AlignLeft);
+    deviceLayout->addWidget(m_soundShow, 0, Qt::AlignRight);
+
+    BackgroundWidget *deviceWidget = new BackgroundWidget(this);
+    deviceWidget->setLayout(deviceLayout);
+
+    // 音量滑动条
+    QHBoxLayout *volumeCtrlLayout = new QHBoxLayout;
+    volumeCtrlLayout->setSpacing(0);
+    volumeCtrlLayout->setMargin(0);
+    volumeCtrlLayout->setContentsMargins(12, 0, 12, 0);
+    volumeCtrlLayout->addWidget(m_volumeIconMin);
+    volumeCtrlLayout->addWidget(m_volumeSlider);
+    volumeCtrlLayout->addWidget(m_volumeIconMax);
+
+    BackgroundWidget *volumnWidget = new BackgroundWidget(this);
+    volumnWidget->setLayout(volumeCtrlLayout);
+
+    m_centralLayout = new QVBoxLayout(this);
+    m_centralLayout->setContentsMargins(0, 0, 0, 0);
+    m_centralLayout->setMargin(0);
+    m_centralLayout->setSpacing(0);
+    m_centralLayout->addWidget(deviceWidget);
+    m_centralLayout->addWidget(m_seperator);
+    m_centralLayout->addWidget(volumnWidget);
+    // 需要判断是否有声音端口
+    m_centralLayout->addWidget(m_secondSeperator);
+
+    m_centralLayout->addWidget(m_listView);
+    m_centralWidget->setLayout(m_centralLayout);
+    m_centralWidget->setFixedWidth(WIDTH);
+    m_centralWidget->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
+
+    setFixedWidth(WIDTH);
+    setWidget(m_centralWidget);
+    setContentsMargins(0, 0, 0, 0);
+    setFrameShape(QFrame::NoFrame);
+    setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+    setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+    m_centralWidget->setAutoFillBackground(false);
+    viewport()->setAutoFillBackground(false);
+    m_listView->setItemDelegate(new DStyledItemDelegate(m_listView));
+
+    m_secondSeperator->setVisible(m_model->rowCount() > 1);
+
     updateVolumeSliderStatus(Utils::SettingValue("com.deepin.dde.dock.module.sound", QByteArray(), "Enabled").toString());
     connect(m_gsettings, &QGSettings::changed, [ = ] (const QString &key) {
         if (key == GSETTING_SOUND_OUTPUT_SLIDER) {
             updateVolumeSliderStatus(m_gsettings->get(GSETTING_SOUND_OUTPUT_SLIDER).toString());
         }
     });
-
-    m_centralLayout = new QVBoxLayout;
-    m_centralLayout->addLayout(deviceLineLayout);
-    m_centralLayout->addSpacing(8);
-    m_centralLayout->addLayout(volumeCtrlLayout);
-    m_centralLayout->setContentsMargins(8, 8, 8, 0);
-
-    m_listView->setModel(m_model);
-    m_centralLayout->addWidget(m_listView);
-    m_centralWidget->setLayout(m_centralLayout);
-    m_centralWidget->setFixedWidth(WIDTH);
-    m_centralWidget->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Preferred);
-
-    setFixedWidth(WIDTH);
-    setWidget(m_centralWidget);
-    setFrameShape(QFrame::NoFrame);
-    setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
-    setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
-    m_centralWidget->setAutoFillBackground(false);
-    viewport()->setAutoFillBackground(false);
-
     connect(qApp, &QGuiApplication::fontChanged, this, &SoundApplet::updateListHeight);
-    connect(m_volumeBtn, &DIconButton::clicked, this, &SoundApplet::toggleMute);
     connect(m_volumeSlider, &VolumeSlider::valueChanged, this, &SoundApplet::volumeSliderValueChanged);
-    connect(m_volumeSlider, &VolumeSlider::requestPlaySoundEffect, this, &SoundApplet::onPlaySoundEffect);
-    connect(m_audioInter, &DBusAudio::DefaultSinkChanged, this, static_cast<void (SoundApplet::*)()>(&SoundApplet::defaultSinkChanged));
+    connect(m_audioInter, &DBusAudio::DefaultSinkChanged, this, &SoundApplet::onDefaultSinkChanged);
     connect(m_audioInter, &DBusAudio::IncreaseVolumeChanged, this, &SoundApplet::increaseVolumeChanged);
     connect(m_audioInter, &DBusAudio::PortEnabledChanged, [this](uint cardId, QString portId) {
-            portEnableChange(cardId, portId);
+        portEnableChange(cardId, portId);
     });;
     connect(m_listView, &DListView::clicked, this, [this](const QModelIndex & idx) {
         const Port * port = m_listView->model()->data(idx, Qt::WhatsThisPropertyRole).value<const Port *>();
@@ -234,7 +248,7 @@ void SoundApplet::initUi()
     QDBusConnection::sessionBus().connect("com.deepin.daemon.Audio", "/com/deepin/daemon/Audio", "org.freedesktop.DBus.Properties"
                                           ,"PropertiesChanged", "sa{sv}as", this, SLOT(haldleDbusSignal(QDBusMessage)));
 
-    QMetaObject::invokeMethod(this, "defaultSinkChanged", Qt::QueuedConnection);
+    QMetaObject::invokeMethod(this, "onDefaultSinkChanged", Qt::QueuedConnection);
 
     refreshIcon();
 
@@ -256,7 +270,7 @@ VolumeSlider *SoundApplet::mainSlider()
     return m_volumeSlider;
 }
 
-void SoundApplet::defaultSinkChanged()
+void SoundApplet::onDefaultSinkChanged()
 {
     //防止手动切换设备，与后端交互时，获取到多个信号，设备切换多次，造成混乱
     QThread::msleep(200);
@@ -282,14 +296,14 @@ void SoundApplet::defaultSinkChanged()
     }
     activePort(portId,cardId);
 
+    //无声卡状态下，会有伪sink设备，显示音量为0
+    onVolumeChanged(findPort(portId, cardId) != nullptr ? m_defSinkInter->volume() : 0);
     emit defaultSinkChanged(m_defSinkInter);
-    onVolumeChanged(m_defSinkInter->volume());
 }
 
 void SoundApplet::onVolumeChanged(double volume)
 {
-    m_volumeSlider->setValue(static_cast<int>(std::min(150.0, volume * 100.0)));
-
+    m_volumeSlider->setValue(std::min(150, qRound(volume * 100.0)));
     m_soundShow->setText(QString::number(volume * 100) + '%');
     emit volumeChanged(m_volumeSlider->value());
     refreshIcon();
@@ -302,25 +316,6 @@ void SoundApplet::volumeSliderValueChanged()
         m_defSinkInter->SetMuteQueued(false);
 }
 
-void SoundApplet::sinkInputsChanged()
-{
-    m_centralWidget->setVisible(false);
-    QVBoxLayout *appLayout = m_centralLayout;
-    while (QLayoutItem *item = appLayout->takeAt(4)) {
-        delete item->widget();
-        delete item;
-    }
-
-    for (auto input : m_audioInter->sinkInputs()) {
-        appLayout->addWidget(new HorizontalSeparator);
-
-        SinkInputWidget *si = new SinkInputWidget(input.path());
-        appLayout->addWidget(si);
-    }
-
-    m_centralWidget->setVisible(true);
-}
-
 void SoundApplet::cardsChanged(const QString &cards)
 {
     QMap<uint, QStringList> tmpCardIds;
@@ -362,8 +357,8 @@ void SoundApplet::cardsChanged(const QString &cards)
         tmpCardIds.insert(cardId, tmpPorts);
     }
 
-    defaultSinkChanged();//重新获取切换的设备信息
-    enableDevice(true);
+    onDefaultSinkChanged();//重新获取切换的设备信息
+    enableDevice(existActiveOutputDevice());
 
     for (Port *port : m_ports) {
         //只要有一个设备在控制中心被禁用后，在任务栏声音设备列表中该设备会被移除，
@@ -384,19 +379,18 @@ void SoundApplet::cardsChanged(const QString &cards)
     removeLastDevice();
     updateListHeight();
 }
-void SoundApplet::toggleMute()
-{
-    m_defSinkInter->SetMuteQueued(!m_defSinkInter->mute());
-}
-
-void SoundApplet::onPlaySoundEffect()
-{
-
-}
 
 void SoundApplet::increaseVolumeChanged()
 {
+    // 触发VolumeSlider::valueChanged
     m_volumeSlider->setMaximum(m_audioInter->maxUIVolume() * 100.0f);
+
+    // onDefaultSinkChanged()比increaseVolumeChanged()先执行，设置完最大值后需要重新设置当前值
+    if (m_defSinkInter) {
+        //　规避因开启声音增强，声音值超过100后，关闭声音增强后读取到数据为旧数据的问题
+        if (m_audioInter->increaseVolume())
+            m_volumeSlider->setValue(std::min(150, qRound(m_defSinkInter->volume() * 100.0)));
+    }
 }
 
 void SoundApplet::refreshIcon()
@@ -404,7 +398,7 @@ void SoundApplet::refreshIcon()
     if (!m_defSinkInter)
         return;
 
-    const bool mute = m_defSinkInter->mute();
+    const bool mute = existActiveOutputDevice() ? m_defSinkInter->mute() : true;
 
     QString volumeString;
 
@@ -417,28 +411,17 @@ void SoundApplet::refreshIcon()
     QString iconLeft = QString("audio-volume-%1-symbolic").arg(volumeString);
     QString iconRight = QString("audio-volume-high-symbolic");
 
-    QColor color;
-    switch (DGuiApplicationHelper::instance()->themeType()) {
-    case DGuiApplicationHelper::LightType:
-        color = Qt::black;
+    if (DGuiApplicationHelper::instance()->themeType() == DGuiApplicationHelper::LightType) {
         iconLeft.append("-dark");
         iconRight.append("-dark");
-        break;
-    default:
-        color = Qt::white;
-        break;
-    }
-    //主题改变时，同步修改item颜色
-    for (int i = 0; i < m_model->rowCount(); i++) {
-        auto item = m_model->item(i);
-        item->setForeground(color);
     }
+
     const auto ratio = devicePixelRatioF();
     QPixmap ret = ImageUtil::loadSvg(iconRight, ":/", ICON_SIZE, ratio);
     m_volumeIconMax->setPixmap(ret);
 
     ret = ImageUtil::loadSvg(iconLeft, ":/", ICON_SIZE, ratio);
-    m_volumeBtn->setIcon(ret);
+    m_volumeIconMin->setPixmap(ret);
 }
 
 /**
@@ -475,10 +458,12 @@ bool SoundApplet::containsPort(const Port *port)
 
 Port *SoundApplet::findPort(const QString &portId, const uint &cardId) const
 {
-    for (Port *port : m_ports) {
-        if (port->id() == portId && port->cardId() == cardId) {
-            return port;
-        }
+    auto it = std::find_if(m_ports.begin(), m_ports.end(), [ = ] (Port *p) {
+        return (p->id() == portId && p->cardId() == cardId);
+    });
+
+    if (it != m_ports.end()) {
+        return *it;
     }
 
     return nullptr;
@@ -489,8 +474,7 @@ void SoundApplet::addPort(const Port *port)
     DStandardItem *pi = new DStandardItem;
     QString deviceName = port->name() + "(" + port->cardName() + ")";
     pi->setText(deviceName);
-    pi->setBackground(Qt::transparent);
-    pi->setForeground(QBrush(Qt::black));
+    pi->setTextColorRole(QPalette::BrightText);
     pi->setData(QVariant::fromValue<const Port *>(port), Qt::WhatsThisPropertyRole);
 
     connect(port, &Port::nameChanged, this, [ = ](const QString &str) {
@@ -508,8 +492,11 @@ void SoundApplet::addPort(const Port *port)
     if (port->isActive()) {
         pi->setCheckState(Qt::CheckState::Checked);
     }
+
     m_model->appendRow(pi);
     m_model->sort(0);
+    m_secondSeperator->setVisible(m_model->rowCount() > 1);
+    updateListHeight();
 }
 
 void SoundApplet::removePort(const QString &portId, const uint &cardId)
@@ -528,6 +515,8 @@ void SoundApplet::removePort(const QString &portId, const uint &cardId)
     };
 
     rmFunc(m_model);
+    m_secondSeperator->setVisible(m_model->rowCount() > 1);
+    updateListHeight();
 }
 
 /**
@@ -550,8 +539,7 @@ void SoundApplet::activePort(const QString &portId, const uint &cardId)
 
 void SoundApplet::updateCradsInfo()
 {
-    QDBusInterface inter("com.deepin.daemon.Audio", "/com/deepin/daemon/Audio","com.deepin.daemon.Audio",QDBusConnection::sessionBus(), this);
-    QString info = inter.property("CardsWithoutUnavailable").toString();
+    QString info = m_audioInter->property("CardsWithoutUnavailable").toString();
     if(m_deviceInfo != info){
         cardsChanged(info);
         m_deviceInfo = info;
@@ -566,7 +554,7 @@ void SoundApplet::enableDevice(bool flag)
     } else if ("Enabled" == status) {
         m_volumeSlider->setEnabled(flag);
     }
-    m_volumeBtn->setEnabled(flag);
+    m_volumeIconMin->setEnabled(flag);
     m_soundShow->setEnabled(flag);
     m_volumeIconMax->setEnabled(flag);
     m_deviceLabel->setEnabled(flag);
@@ -621,15 +609,53 @@ void SoundApplet::updateVolumeSliderStatus(const QString &status)
         flag = false;
     }
     m_volumeSlider->setEnabled(flag);
-    m_volumeBtn->setEnabled(flag);
+    m_volumeIconMin->setEnabled(flag);
     m_volumeIconMax->setEnabled(flag);
 
     flag = "Hiden" != status;
     m_volumeSlider->setVisible(flag);
-    m_volumeBtn->setVisible(flag);
+    m_volumeIconMin->setVisible(flag);
     m_volumeIconMax->setVisible(flag);
 }
 
+/** 判断是否存在未禁用的声音输出设备
+ * @brief SoundApplet::existActiveOutputDevice
+ * @return 存在返回true,否则返回false
+ */
+bool SoundApplet::existActiveOutputDevice()
+{
+    QString info = m_audioInter->property("CardsWithoutUnavailable").toString();
+
+    QJsonDocument doc = QJsonDocument::fromJson(info.toUtf8());
+    QJsonArray jCards = doc.array();
+    for (QJsonValue cV : jCards) {
+        QJsonObject jCard = cV.toObject();
+        QJsonArray jPorts = jCard["Ports"].toArray();
+
+        for (QJsonValue pV : jPorts) {
+            QJsonObject jPort = pV.toObject();
+            if (jPort["Direction"].toInt() == 1 && jPort["Enabled"].toBool())
+                return true;
+        }
+    }
+
+    return false;
+}
+
+bool SoundApplet::eventFilter(QObject *watcher, QEvent *event)
+{
+    // 当控制中心禁用所有输出设备时，静音按钮置灰，其他情况正常．
+    if (watcher == m_volumeIconMin && event->type() == QEvent::MouseButtonRelease) {
+        if (!existActiveOutputDevice()) {
+            m_volumeIconMin->setEnabled(false);
+        } else {
+            m_volumeIconMin->setEnabled(true);
+            m_defSinkInter->SetMuteQueued(!m_defSinkInter->mute());
+        }
+    }
+    return false;
+}
+
 void SoundApplet::haldleDbusSignal(const QDBusMessage &msg)
 {
     Q_UNUSED(msg)
@@ -640,7 +666,8 @@ void SoundApplet::haldleDbusSignal(const QDBusMessage &msg)
 void SoundApplet::updateListHeight()
 {
     //设备数多于10个时显示滚动条,固定高度
-    int count = m_model->rowCount();
+    int count = m_model->rowCount() == 1 ? 0 : m_model->rowCount();
+
     if (m_model->rowCount() > 10) {
         count = 10;
         m_listView->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
@@ -654,17 +681,17 @@ void SoundApplet::updateListHeight()
 
     int listMargin = m_listView->contentsMargins().top() + m_listView->contentsMargins().bottom();
     //显示声音设备列表高度 = 设备的高度 + 间隔 + 边距
-    int viewHeight = visualHeight + m_listView->spacing() * count * 2 + listMargin;
+    int viewHeight = visualHeight + m_listView->spacing() * (count - 1) + listMargin;
     // 设备信息高度 = 设备标签 + 分隔线 + 滚动条 + 间隔
     int labelHeight = m_deviceLabel->height() > m_soundShow->height() ? m_deviceLabel->height() : m_soundShow->height();
-    int infoHeight = labelHeight + m_separator->height() + m_volumeSlider->height() + m_centralLayout->spacing() * 3 + DEVICE_SPACING;
+    int infoHeight = labelHeight + m_seperator->height() * 2 + m_volumeSlider->height();
     int margain = m_centralLayout->contentsMargins().top() + m_centralLayout->contentsMargins().bottom();
     //整个界面高度 = 显示声音设备列表高度 + 设备信息高度 + 边距
     int totalHeight = viewHeight + infoHeight + margain;
+    //加上分割线占用的高度，否则显示界面高度不够显示，会造成音频列表item最后一项比其它项的高度小
     m_listView->setFixedHeight(viewHeight);
     setFixedHeight(totalHeight);
     m_centralWidget->setFixedHeight(totalHeight);
-    update();
 }
 
 void SoundApplet::portEnableChange(unsigned int cardId, QString portId)
