@@ -25,11 +25,15 @@
 
 #include <dbusmenu-qt5/dbusmenuimporter.h>
 
+#include <DGuiApplicationHelper>
+
 #include <QPainter>
 #include <QApplication>
 
 #include <xcb/xproto.h>
 
+DGUI_USE_NAMESPACE
+
 #define IconSize 20
 
 const QStringList ItemCategoryList {"ApplicationStatus", "Communications", "SystemServices", "Hardware"};
@@ -68,6 +72,7 @@ SNITrayWidget::SNITrayWidget(const QString &sniServicePath, QWidget *parent)
         arrowRectangle->setArrowHeight(10);
         arrowRectangle->setObjectName("snitraypopup");
         PopupWindow = arrowRectangle;
+        connect(qApp, &QApplication::aboutToQuit, PopupWindow, &DockPopupWindow::deleteLater);
     }
 
     if (m_sniServicePath.startsWith("/") || !m_sniServicePath.contains("/")) {
@@ -79,8 +84,10 @@ SNITrayWidget::SNITrayWidget(const QString &sniServicePath, QWidget *parent)
     m_dbusService = pair.first;
     m_dbusPath = pair.second;
 
+    QDBusConnection conn = QDBusConnection::sessionBus();
+    setOwnerPID(conn.interface()->servicePid(m_dbusService));
+
     m_sniInter = new StatusNotifierItem(m_dbusService, m_dbusPath, QDBusConnection::sessionBus(), this);
-    m_sniInter->setSync(false);
 
     if (!m_sniInter->isValid()) {
         qDebug() << "SNI dbus interface is invalid!" << m_dbusService << m_dbusPath << m_sniInter->lastError();
@@ -94,6 +101,7 @@ SNITrayWidget::SNITrayWidget(const QString &sniServicePath, QWidget *parent)
     m_updateAttentionIconTimer->setInterval(1000);
     m_updateAttentionIconTimer->setSingleShot(true);
 
+    connect(DGuiApplicationHelper::instance(), &DGuiApplicationHelper::themeTypeChanged, this, &SNITrayWidget::refreshIcon);
     connect(m_updateIconTimer, &QTimer::timeout, this, &SNITrayWidget::refreshIcon);
     connect(m_updateOverlayIconTimer, &QTimer::timeout, this, &SNITrayWidget::refreshOverlayIcon);
     connect(m_updateAttentionIconTimer, &QTimer::timeout, this, &SNITrayWidget::refreshAttentionIcon);
@@ -167,10 +175,6 @@ QString SNITrayWidget::itemKeyForConfig()
     return QString("sni:%1").arg(key);
 }
 
-void SNITrayWidget::setActive(const bool active)
-{
-}
-
 void SNITrayWidget::updateIcon()
 {
     m_updateIconTimer->start();
@@ -199,11 +203,6 @@ void SNITrayWidget::sendClick(uint8_t mouseButton, int x, int y)
     }
 }
 
-const QImage SNITrayWidget::trayImage()
-{
-    return m_pixmap.toImage();
-}
-
 bool SNITrayWidget::isValid()
 {
     return m_sniInter->isValid();
@@ -252,6 +251,13 @@ QPair<QString, QString> SNITrayWidget::serviceAndPath(const QString &servicePath
     return pair;
 }
 
+uint SNITrayWidget::servicePID(const QString &servicePath)
+{
+    QString serviceName = serviceAndPath(servicePath).first;
+    QDBusConnection conn = QDBusConnection::sessionBus();
+    return conn.interface()->servicePid(serviceName);
+}
+
 void SNITrayWidget::initSNIPropertys()
 {
     m_sniAttentionIconName = m_sniInter->attentionIconName();
@@ -542,7 +548,7 @@ QPixmap SNITrayWidget::newIconPixmap(IconType iconType)
         // so, it should be the last fallback
         if (!iconName.isEmpty()) {
             // ThemeAppIcon::getIcon 会处理高分屏缩放问题
-            pixmap = ThemeAppIcon::getIcon(iconName, IconSize, devicePixelRatioF());
+             ThemeAppIcon::getIcon(pixmap, iconName, IconSize);
             if (!pixmap.isNull()) {
                 break;
             }
@@ -665,7 +671,12 @@ void SNITrayWidget::showHoverTips()
         if (tooltip.title.isEmpty())
             return;
 
-        m_tipsLabel->setText(tooltip.title);
+        // 当提示信息中有换行符时，需要使用setTextList
+        if (tooltip.title.contains('\n'))
+            m_tipsLabel->setTextList(tooltip.title.split('\n'));
+        else
+            m_tipsLabel->setText(tooltip.title);
+
         m_tipsLabel->setAccessibleName(itemKeyForConfig().replace("sni:",""));
 
         showPopupWindow(m_tipsLabel);
