@@ -24,6 +24,7 @@
 #include "fashiontray/fashiontrayitem.h"
 #include "snitraywidget.h"
 #include "utils.h"
+#include "../widgets/tipswidget.h"
 
 #include <QDir>
 #include <QWindow>
@@ -33,8 +34,8 @@
 #include <QFuture>
 #include <QFutureWatcher>
 
-#include "../widgets/tipswidget.h"
-#include "xcb/xcb_icccm.h"
+#include <xcb/xcb_icccm.h>
+#include <X11/Xlib.h>
 
 #define PLUGIN_ENABLED_KEY "enable"
 #define FASHION_MODE_TRAYS_SORTED   "fashion-mode-trays-sorted"
@@ -42,12 +43,23 @@
 #define SNI_WATCHER_SERVICE "org.kde.StatusNotifierWatcher"
 #define SNI_WATCHER_PATH "/StatusNotifierWatcher"
 
+#define REGISTERTED_WAY_IS_SNI 1
+#define REGISTERTED_WAY_IS_XEMBED 2
+
 using org::kde::StatusNotifierWatcher;
 using namespace Dock;
+
 TrayPlugin::TrayPlugin(QObject *parent)
     : QObject(parent)
     , m_pluginLoaded(false)
+    , xcb_connection(nullptr)
+    , m_display(nullptr)
 {
+    if (Utils::IS_WAYLAND_DISPLAY) {
+        int screenp = 0;
+        xcb_connection = xcb_connect(qgetenv("DISPLAY"), &screenp);
+        m_display = XOpenDisplay(nullptr);
+    }
 }
 
 const QString TrayPlugin::pluginName() const
@@ -84,7 +96,6 @@ void TrayPlugin::init(PluginProxyInterface *proxyInter)
     m_systemTraysController = new SystemTraysController(this);
     m_refreshXEmbedItemsTimer = new QTimer(this);
     m_refreshSNIItemsTimer = new QTimer(this);
-    m_tipsLabel = new TipsWidget;
 
     m_refreshXEmbedItemsTimer->setInterval(0);
     m_refreshXEmbedItemsTimer->setSingleShot(true);
@@ -92,10 +103,6 @@ void TrayPlugin::init(PluginProxyInterface *proxyInter)
     m_refreshSNIItemsTimer->setInterval(0);
     m_refreshSNIItemsTimer->setSingleShot(true);
 
-    m_tipsLabel->setObjectName("tray");
-    m_tipsLabel->setText(tr("System Tray"));
-    m_tipsLabel->setVisible(false);
-
     connect(m_systemTraysController, &SystemTraysController::pluginItemAdded, this, &TrayPlugin::addTrayWidget);
     connect(m_systemTraysController, &SystemTraysController::pluginItemRemoved, this, [ = ](const QString & itemKey) { trayRemoved(itemKey); });
 
@@ -103,6 +110,7 @@ void TrayPlugin::init(PluginProxyInterface *proxyInter)
 
     switchToMode(displayMode());
 
+    // 加载sni，xem，自定义indicator协议以及其他托盘插件
     QTimer::singleShot(0, this, &TrayPlugin::loadIndicator);
     QTimer::singleShot(0, m_systemTraysController, &SystemTraysController::startLoader);
     QTimer::singleShot(0, this, &TrayPlugin::initSNI);
@@ -126,6 +134,8 @@ bool TrayPlugin::pluginIsDisable()
 
 void TrayPlugin::displayModeChanged(const Dock::DisplayMode mode)
 {
+    Q_UNUSED(mode);
+
     if (pluginIsDisable()) {
         return;
     }
@@ -302,6 +312,10 @@ void TrayPlugin::initXEmbed()
     m_refreshXEmbedItemsTimer->start();
 }
 
+/**
+ * @brief TrayPlugin::initSNI
+ * @note 初始化监听信号绑定
+ */
 void TrayPlugin::initSNI()
 {
     connect(m_refreshSNIItemsTimer, &QTimer::timeout, this, &TrayPlugin::sniItemsChanged);
@@ -311,6 +325,10 @@ void TrayPlugin::initSNI()
     m_refreshSNIItemsTimer->start();
 }
 
+/**
+ * @brief TrayPlugin::sniItemsChanged
+ * @note 移除关闭的item,插入新增item
+ */
 void TrayPlugin::sniItemsChanged()
 {
     const QStringList &itemServicePaths = m_sniWatcher->registeredStatusNotifierItems();
@@ -321,6 +339,7 @@ void TrayPlugin::sniItemsChanged()
     }
     for (auto itemKey : m_trayMap.keys()) {
         if (!sinTrayKeyList.contains(itemKey) && SNITrayWidget::isSNIKey(itemKey)) {
+            m_registertedPID.take(m_trayMap[itemKey]->getOwnerPID());
             trayRemoved(itemKey);
         }
     }
@@ -330,29 +349,42 @@ void TrayPlugin::sniItemsChanged()
             m_passiveSNITrayMap.take(itemKey)->deleteLater();
         }
     }
-
     for (int i = 0; i < sinTrayKeyList.size(); ++i) {
-        traySNIAdded(sinTrayKeyList.at(i), itemServicePaths.at(i));
+        uint pid = SNITrayWidget::servicePID(itemServicePaths.at(i));
+        if (m_registertedPID.value(pid, REGISTERTED_WAY_IS_SNI) == REGISTERTED_WAY_IS_SNI) {
+            traySNIAdded(sinTrayKeyList.at(i), itemServicePaths.at(i));
+            m_registertedPID.insert(pid, REGISTERTED_WAY_IS_SNI);
+        }
     }
 }
 
 void TrayPlugin::xembedItemsChanged()
 {
     QList<quint32> winidList = m_trayInter->trayIcons();
-    QStringList trayKeyList;
+    QStringList newlyAddedTrayKeyList;
+    QStringList allKeytary;
+    QList<quint32> newlyAddedWindowID;
 
     for (auto winid : winidList) {
-        trayKeyList << XEmbedTrayWidget::toXEmbedKey(winid);
+        uint pid = XEmbedTrayWidget::getWindowPID(winid);
+        allKeytary << XEmbedTrayWidget::toXEmbedKey(winid);
+
+        if (m_registertedPID.value(pid, REGISTERTED_WAY_IS_XEMBED) == REGISTERTED_WAY_IS_XEMBED) {
+            m_registertedPID.insert(pid, REGISTERTED_WAY_IS_XEMBED);
+            newlyAddedWindowID << winid;
+            newlyAddedTrayKeyList << XEmbedTrayWidget::toXEmbedKey(winid);
+        }
     }
 
     for (auto tray : m_trayMap.keys()) {
-        if (!trayKeyList.contains(tray) && XEmbedTrayWidget::isXEmbedKey(tray)) {
+        if (!allKeytary.contains(tray) && XEmbedTrayWidget::isXEmbedKey(tray)) {
+            m_registertedPID.take(m_trayMap[tray]->getOwnerPID());
             trayRemoved(tray);
         }
     }
 
-    for (int i = 0; i < trayKeyList.size(); ++i) {
-        trayXEmbedAdded(trayKeyList.at(i), winidList.at(i));
+    for (int i = 0; i < newlyAddedTrayKeyList.size(); ++i) {
+        trayXEmbedAdded(newlyAddedTrayKeyList.at(i), newlyAddedWindowID.at(i));
     }
 }
 
@@ -388,7 +420,7 @@ void TrayPlugin::trayXEmbedAdded(const QString &itemKey, quint32 winId)
     if (!Utils::SettingValue("com.deepin.dde.dock.module.systemtray", QByteArray(), "enable", false).toBool())
         return;
 
-    AbstractTrayWidget *trayWidget = new XEmbedTrayWidget(winId);
+    AbstractTrayWidget *trayWidget = Utils::IS_WAYLAND_DISPLAY ? new XEmbedTrayWidget(winId, xcb_connection, m_display) : new XEmbedTrayWidget(winId);
     if (trayWidget->isValid())
         addTrayWidget(itemKey, trayWidget);
     else {
@@ -399,6 +431,7 @@ void TrayPlugin::trayXEmbedAdded(const QString &itemKey, quint32 winId)
 void TrayPlugin::traySNIAdded(const QString &itemKey, const QString &sniServicePath)
 {
     QFutureWatcher<bool> *watcher = new QFutureWatcher<bool>();
+    // 开线程去处理添加任务
     connect(watcher, &QFutureWatcher<int>::finished, this, [=] {
         watcher->deleteLater();
         if (!watcher->result()) {
@@ -471,7 +504,7 @@ void TrayPlugin::trayIndicatorAdded(const QString &itemKey, const QString &indic
 
     IndicatorTray *indicatorTray = nullptr;
     if (!m_indicatorMap.keys().contains(indicatorName)) {
-        indicatorTray = new IndicatorTray(indicatorName);
+        indicatorTray = new IndicatorTray(indicatorName, this);
         m_indicatorMap[indicatorName] = indicatorTray;
     } else {
         indicatorTray = m_indicatorMap[itemKey];
